# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.deal_details import DealDetails  # noqa: F401,E501
from paapi5_python_sdk.models.offer_availability_v2 import OfferAvailabilityV2  # noqa: F401,E501
from paapi5_python_sdk.models.offer_condition_v2 import OfferConditionV2  # noqa: F401,E501
from paapi5_python_sdk.models.offer_loyalty_points_v2 import OfferLoyaltyPointsV2  # noqa: F401,E501
from paapi5_python_sdk.models.offer_merchant_info_v2 import OfferMerchantInfoV2  # noqa: F401,E501
from paapi5_python_sdk.models.offer_price_v2 import OfferPriceV2  # noqa: F401,E501
from paapi5_python_sdk.models.offer_type import OfferType  # noqa: F401,E501


class OfferListingV2(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "availability": "OfferAvailabilityV2",
        "condition": "OfferConditionV2",
        "deal_details": "DealDetails",
        "is_buy_box_winner": "bool",
        "loyalty_points": "OfferLoyaltyPointsV2",
        "merchant_info": "OfferMerchantInfoV2",
        "price": "OfferPriceV2",
        "type": "OfferType",
        "violates_map": "bool",
    }

    attribute_map = {
        "availability": "Availability",
        "condition": "Condition",
        "deal_details": "DealDetails",
        "is_buy_box_winner": "IsBuyBoxWinner",
        "loyalty_points": "LoyaltyPoints",
        "merchant_info": "MerchantInfo",
        "price": "Price",
        "type": "Type",
        "violates_map": "ViolatesMAP",
    }

    def __init__(
        self,
        availability=None,
        condition=None,
        deal_details=None,
        is_buy_box_winner=None,
        loyalty_points=None,
        merchant_info=None,
        price=None,
        type=None,
        violates_map=None,
    ):  # noqa: E501
        """OfferListingV2 - a model defined in Swagger"""  # noqa: E501

        self._availability = None
        self._condition = None
        self._deal_details = None
        self._is_buy_box_winner = None
        self._loyalty_points = None
        self._merchant_info = None
        self._price = None
        self._type = None
        self._violates_map = None
        self.discriminator = None

        if availability is not None:
            self.availability = availability
        if condition is not None:
            self.condition = condition
        if deal_details is not None:
            self.deal_details = deal_details
        if is_buy_box_winner is not None:
            self.is_buy_box_winner = is_buy_box_winner
        if loyalty_points is not None:
            self.loyalty_points = loyalty_points
        if merchant_info is not None:
            self.merchant_info = merchant_info
        if price is not None:
            self.price = price
        if type is not None:
            self.type = type
        if violates_map is not None:
            self.violates_map = violates_map

    @property
    def availability(self):
        """Gets the availability of this OfferListingV2.  # noqa: E501


        :return: The availability of this OfferListingV2.  # noqa: E501
        :rtype: OfferAvailabilityV2
        """
        return self._availability

    @availability.setter
    def availability(self, availability):
        """Sets the availability of this OfferListingV2.


        :param availability: The availability of this OfferListingV2.  # noqa: E501
        :type: OfferAvailabilityV2
        """

        self._availability = availability

    @property
    def condition(self):
        """Gets the condition of this OfferListingV2.  # noqa: E501


        :return: The condition of this OfferListingV2.  # noqa: E501
        :rtype: OfferConditionV2
        """
        return self._condition

    @condition.setter
    def condition(self, condition):
        """Sets the condition of this OfferListingV2.


        :param condition: The condition of this OfferListingV2.  # noqa: E501
        :type: OfferConditionV2
        """

        self._condition = condition

    @property
    def deal_details(self):
        """Gets the deal_details of this OfferListingV2.  # noqa: E501


        :return: The deal_details of this OfferListingV2.  # noqa: E501
        :rtype: DealDetails
        """
        return self._deal_details

    @deal_details.setter
    def deal_details(self, deal_details):
        """Sets the deal_details of this OfferListingV2.


        :param deal_details: The deal_details of this OfferListingV2.  # noqa: E501
        :type: DealDetails
        """

        self._deal_details = deal_details

    @property
    def is_buy_box_winner(self):
        """Gets the is_buy_box_winner of this OfferListingV2.  # noqa: E501


        :return: The is_buy_box_winner of this OfferListingV2.  # noqa: E501
        :rtype: bool
        """
        return self._is_buy_box_winner

    @is_buy_box_winner.setter
    def is_buy_box_winner(self, is_buy_box_winner):
        """Sets the is_buy_box_winner of this OfferListingV2.


        :param is_buy_box_winner: The is_buy_box_winner of this OfferListingV2.  # noqa: E501
        :type: bool
        """

        self._is_buy_box_winner = is_buy_box_winner

    @property
    def loyalty_points(self):
        """Gets the loyalty_points of this OfferListingV2.  # noqa: E501


        :return: The loyalty_points of this OfferListingV2.  # noqa: E501
        :rtype: OfferLoyaltyPointsV2
        """
        return self._loyalty_points

    @loyalty_points.setter
    def loyalty_points(self, loyalty_points):
        """Sets the loyalty_points of this OfferListingV2.


        :param loyalty_points: The loyalty_points of this OfferListingV2.  # noqa: E501
        :type: OfferLoyaltyPointsV2
        """

        self._loyalty_points = loyalty_points

    @property
    def merchant_info(self):
        """Gets the merchant_info of this OfferListingV2.  # noqa: E501


        :return: The merchant_info of this OfferListingV2.  # noqa: E501
        :rtype: OfferMerchantInfoV2
        """
        return self._merchant_info

    @merchant_info.setter
    def merchant_info(self, merchant_info):
        """Sets the merchant_info of this OfferListingV2.


        :param merchant_info: The merchant_info of this OfferListingV2.  # noqa: E501
        :type: OfferMerchantInfoV2
        """

        self._merchant_info = merchant_info

    @property
    def price(self):
        """Gets the price of this OfferListingV2.  # noqa: E501


        :return: The price of this OfferListingV2.  # noqa: E501
        :rtype: OfferPriceV2
        """
        return self._price

    @price.setter
    def price(self, price):
        """Sets the price of this OfferListingV2.


        :param price: The price of this OfferListingV2.  # noqa: E501
        :type: OfferPriceV2
        """

        self._price = price

    @property
    def type(self):
        """Gets the type of this OfferListingV2.  # noqa: E501


        :return: The type of this OfferListingV2.  # noqa: E501
        :rtype: OfferType
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this OfferListingV2.


        :param type: The type of this OfferListingV2.  # noqa: E501
        :type: OfferType
        """

        self._type = type

    @property
    def violates_map(self):
        """Gets the violates_map of this OfferListingV2.  # noqa: E501


        :return: The violates_map of this OfferListingV2.  # noqa: E501
        :rtype: bool
        """
        return self._violates_map

    @violates_map.setter
    def violates_map(self, violates_map):
        """Sets the violates_map of this OfferListingV2.


        :param violates_map: The violates_map of this OfferListingV2.  # noqa: E501
        :type: bool
        """

        self._violates_map = violates_map

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(OfferListingV2, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OfferListingV2):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import logging
import time
import random
from typing import Callable, TypeVar, Any

from paapi5_python_sdk.rest import ApiException

logger = logging.getLogger(__name__)

T = TypeVar("T")


def call_with_backoff(
    func: Callable[..., T],
    *args: Any,
    max_retries: int = 5,
    initial_delay: float = 2.0,
    max_delay: float = 60.0,
    backoff_factor: float = 2.0,
    **kwargs: Any,
) -> T:
    """
    Call an API function with exponential backoff for rate limiting.

    Args:
        func: The API function to call.
        *args: Positional arguments for the function.
        max_retries: Maximum number of retries.
        initial_delay: Initial delay in seconds after a failure.
        max_delay: Maximum delay in seconds.
        backoff_factor: Multiplier for the delay after each failure.
        **kwargs: Keyword arguments for the function.

    Returns:
        The result of the API function call.

    Raises:
        ApiException: If the API call fails after all retries.
    """
    delay = initial_delay
    last_exception = None

    for attempt in range(max_retries + 1):
        try:
            return func(*args, **kwargs)
        except ApiException as e:
            last_exception = e
            # Check for TooManyRequests (HTTP 429) or ThrottlingException
            if e.status == 429 or "TooManyRequests" in str(e) or "Throttling" in str(e):
                if attempt < max_retries:
                    # Add some jitter to prevent thundering herd
                    sleep_time = min(delay, max_delay) * (1 + random.random() * 0.1)
                    logger.warning(
                        f"⚠️  Rate limit hit (429). Retrying in {sleep_time:.2f}s (Attempt {attempt + 1}/{max_retries})..."
                    )
                    time.sleep(sleep_time)
                    delay *= backoff_factor
                else:
                    logger.error("❌ Max retries reached for rate limiting.")
            else:
                # For other errors, re-raise immediately or handle differently?
                # For now, we re-raise non-rate-limit errors to avoid masking real issues.
                # Unless it's a 5xx server error which might be transient.
                if 500 <= e.status < 600:
                    if attempt < max_retries:
                        sleep_time = min(delay, max_delay)
                        logger.warning(
                            f"⚠️  Server error ({e.status}). Retrying in {sleep_time:.2f}s (Attempt {attempt + 1}/{max_retries})..."
                        )
                        time.sleep(sleep_time)
                        delay *= backoff_factor
                        continue

                raise e
        except Exception as e:
            # Unexpected errors
            logger.error(f"Unexpected error in API call: {e}")
            raise e

    if last_exception:
        raise last_exception



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.browse_node_ancestor import BrowseNodeAncestor  # noqa: F401,E501
from paapi5_python_sdk.models.browse_node_child import BrowseNodeChild  # noqa: F401,E501


class BrowseNode(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "ancestor": "BrowseNodeAncestor",
        "children": "list[BrowseNodeChild]",
        "context_free_name": "str",
        "display_name": "str",
        "id": "str",
        "is_root": "bool",
        "sales_rank": "int",
    }

    attribute_map = {
        "ancestor": "Ancestor",
        "children": "Children",
        "context_free_name": "ContextFreeName",
        "display_name": "DisplayName",
        "id": "Id",
        "is_root": "IsRoot",
        "sales_rank": "SalesRank",
    }

    def __init__(
        self,
        ancestor=None,
        children=None,
        context_free_name=None,
        display_name=None,
        id=None,
        is_root=None,
        sales_rank=None,
    ):  # noqa: E501
        """BrowseNode - a model defined in Swagger"""  # noqa: E501

        self._ancestor = None
        self._children = None
        self._context_free_name = None
        self._display_name = None
        self._id = None
        self._is_root = None
        self._sales_rank = None
        self.discriminator = None

        if ancestor is not None:
            self.ancestor = ancestor
        if children is not None:
            self.children = children
        if context_free_name is not None:
            self.context_free_name = context_free_name
        if display_name is not None:
            self.display_name = display_name
        if id is not None:
            self.id = id
        if is_root is not None:
            self.is_root = is_root
        if sales_rank is not None:
            self.sales_rank = sales_rank

    @property
    def ancestor(self):
        """Gets the ancestor of this BrowseNode.  # noqa: E501


        :return: The ancestor of this BrowseNode.  # noqa: E501
        :rtype: BrowseNodeAncestor
        """
        return self._ancestor

    @ancestor.setter
    def ancestor(self, ancestor):
        """Sets the ancestor of this BrowseNode.


        :param ancestor: The ancestor of this BrowseNode.  # noqa: E501
        :type: BrowseNodeAncestor
        """

        self._ancestor = ancestor

    @property
    def children(self):
        """Gets the children of this BrowseNode.  # noqa: E501


        :return: The children of this BrowseNode.  # noqa: E501
        :rtype: list[BrowseNodeChild]
        """
        return self._children

    @children.setter
    def children(self, children):
        """Sets the children of this BrowseNode.


        :param children: The children of this BrowseNode.  # noqa: E501
        :type: list[BrowseNodeChild]
        """

        self._children = children

    @property
    def context_free_name(self):
        """Gets the context_free_name of this BrowseNode.  # noqa: E501


        :return: The context_free_name of this BrowseNode.  # noqa: E501
        :rtype: str
        """
        return self._context_free_name

    @context_free_name.setter
    def context_free_name(self, context_free_name):
        """Sets the context_free_name of this BrowseNode.


        :param context_free_name: The context_free_name of this BrowseNode.  # noqa: E501
        :type: str
        """

        self._context_free_name = context_free_name

    @property
    def display_name(self):
        """Gets the display_name of this BrowseNode.  # noqa: E501


        :return: The display_name of this BrowseNode.  # noqa: E501
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name):
        """Sets the display_name of this BrowseNode.


        :param display_name: The display_name of this BrowseNode.  # noqa: E501
        :type: str
        """

        self._display_name = display_name

    @property
    def id(self):
        """Gets the id of this BrowseNode.  # noqa: E501


        :return: The id of this BrowseNode.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this BrowseNode.


        :param id: The id of this BrowseNode.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def is_root(self):
        """Gets the is_root of this BrowseNode.  # noqa: E501


        :return: The is_root of this BrowseNode.  # noqa: E501
        :rtype: bool
        """
        return self._is_root

    @is_root.setter
    def is_root(self, is_root):
        """Sets the is_root of this BrowseNode.


        :param is_root: The is_root of this BrowseNode.  # noqa: E501
        :type: bool
        """

        self._is_root = is_root

    @property
    def sales_rank(self):
        """Gets the sales_rank of this BrowseNode.  # noqa: E501


        :return: The sales_rank of this BrowseNode.  # noqa: E501
        :rtype: int
        """
        return self._sales_rank

    @sales_rank.setter
    def sales_rank(self, sales_rank):
        """Sets the sales_rank of this BrowseNode.


        :param sales_rank: The sales_rank of this BrowseNode.  # noqa: E501
        :type: int
        """

        self._sales_rank = sales_rank

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(BrowseNode, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BrowseNode):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six


class OfferSavings(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "amount": "float",
        "currency": "str",
        "display_amount": "str",
        "percentage": "int",
        "price_per_unit": "float",
    }

    attribute_map = {
        "amount": "Amount",
        "currency": "Currency",
        "display_amount": "DisplayAmount",
        "percentage": "Percentage",
        "price_per_unit": "PricePerUnit",
    }

    def __init__(
        self,
        amount=None,
        currency=None,
        display_amount=None,
        percentage=None,
        price_per_unit=None,
    ):  # noqa: E501
        """OfferSavings - a model defined in Swagger"""  # noqa: E501

        self._amount = None
        self._currency = None
        self._display_amount = None
        self._percentage = None
        self._price_per_unit = None
        self.discriminator = None

        if amount is not None:
            self.amount = amount
        if currency is not None:
            self.currency = currency
        if display_amount is not None:
            self.display_amount = display_amount
        if percentage is not None:
            self.percentage = percentage
        if price_per_unit is not None:
            self.price_per_unit = price_per_unit

    @property
    def amount(self):
        """Gets the amount of this OfferSavings.  # noqa: E501


        :return: The amount of this OfferSavings.  # noqa: E501
        :rtype: float
        """
        return self._amount

    @amount.setter
    def amount(self, amount):
        """Sets the amount of this OfferSavings.


        :param amount: The amount of this OfferSavings.  # noqa: E501
        :type: float
        """

        self._amount = amount

    @property
    def currency(self):
        """Gets the currency of this OfferSavings.  # noqa: E501


        :return: The currency of this OfferSavings.  # noqa: E501
        :rtype: str
        """
        return self._currency

    @currency.setter
    def currency(self, currency):
        """Sets the currency of this OfferSavings.


        :param currency: The currency of this OfferSavings.  # noqa: E501
        :type: str
        """

        self._currency = currency

    @property
    def display_amount(self):
        """Gets the display_amount of this OfferSavings.  # noqa: E501


        :return: The display_amount of this OfferSavings.  # noqa: E501
        :rtype: str
        """
        return self._display_amount

    @display_amount.setter
    def display_amount(self, display_amount):
        """Sets the display_amount of this OfferSavings.


        :param display_amount: The display_amount of this OfferSavings.  # noqa: E501
        :type: str
        """

        self._display_amount = display_amount

    @property
    def percentage(self):
        """Gets the percentage of this OfferSavings.  # noqa: E501


        :return: The percentage of this OfferSavings.  # noqa: E501
        :rtype: int
        """
        return self._percentage

    @percentage.setter
    def percentage(self, percentage):
        """Sets the percentage of this OfferSavings.


        :param percentage: The percentage of this OfferSavings.  # noqa: E501
        :type: int
        """

        self._percentage = percentage

    @property
    def price_per_unit(self):
        """Gets the price_per_unit of this OfferSavings.  # noqa: E501


        :return: The price_per_unit of this OfferSavings.  # noqa: E501
        :rtype: float
        """
        return self._price_per_unit

    @price_per_unit.setter
    def price_per_unit(self, price_per_unit):
        """Sets the price_per_unit of this OfferSavings.


        :param price_per_unit: The price_per_unit of this OfferSavings.  # noqa: E501
        :type: float
        """

        self._price_per_unit = price_per_unit

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(OfferSavings, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OfferSavings):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Core services for Opportunity Radar."""

from opportunity_core.services.database_manager import (
    DatabaseManager,
    get_db,
    get_db_manager,
    init_database,
)
from opportunity_core.services.deal_detection import DealDetectionEngine, DealType, DetectedDeal
from opportunity_core.services.price_monitoring import PriceMonitoringService
from opportunity_core.services.product_discovery import ProductDiscoveryService

__all__ = [
    "DatabaseManager",
    "init_database",
    "get_db_manager",
    "get_db",
    "ProductDiscoveryService",
    "PriceMonitoringService",
    "DealDetectionEngine",
    "DealType",
    "DetectedDeal",
]



import os
from pathlib import Path

from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict

PROJECT_ROOT = Path(__file__).parent.parent.parent.parent

# Allow custom env file path via environment variable (for dev-once command)
# If ENV_FILE_PATH is set, use it; otherwise use .env
_env_file_path = os.environ.get("ENV_FILE_PATH")
ENV_FILE = PROJECT_ROOT / _env_file_path if _env_file_path else PROJECT_ROOT / ".env"


class PaapiConfig(BaseSettings):
    model_config = SettingsConfigDict(
        env_file=str(ENV_FILE), env_file_encoding="utf-8", case_sensitive=False, extra="ignore"
    )

    access_key: str = Field(alias="PAAPI_ACCESS_KEY")
    secret_key: str = Field(alias="PAAPI_SECRET_KEY")
    partner_tag: str = Field(alias="PAAPI_PARTNER_TAG")
    host: str = Field(alias="PAAPI_HOST")
    region: str = Field(alias="PAAPI_REGION")
    locale: str = Field(alias="PAAPI_LOCALE")
    partner_type: str = Field(alias="PAAPI_PARTNER_TYPE")
    marketplace: str = Field(alias="PAAPI_MARKETPLACE")


class TwitterConfig(BaseSettings):
    """Twitter API configuration for posting deal notifications."""

    model_config = SettingsConfigDict(
        env_file=str(ENV_FILE), env_file_encoding="utf-8", case_sensitive=False, extra="ignore"
    )

    api_key: str = Field(alias="TWITTER_API_KEY")
    api_secret: str = Field(alias="TWITTER_API_SECRET")
    access_token: str = Field(alias="TWITTER_ACCESS_TOKEN")
    access_token_secret: str = Field(alias="TWITTER_ACCESS_TOKEN_SECRET")
    bearer_token: str = Field(default="", alias="TWITTER_BEARER_TOKEN")


class DealConfig(BaseSettings):
    """Configuration for deal detection and monitoring."""

    model_config = SettingsConfigDict(
        env_file=str(ENV_FILE), env_file_encoding="utf-8", case_sensitive=False, extra="ignore"
    )

    min_discount_percentage: int = Field(default=10, alias="MIN_DISCOUNT_PERCENTAGE")
    check_interval_seconds: int = Field(default=300, alias="CHECK_INTERVAL_SECONDS")
    search_keywords: str = Field(default="laptop,telefon,kulaklık", alias="SEARCH_KEYWORDS")
    max_price: float = Field(default=10000.0, alias="MAX_PRICE")


class TelegramConfig(BaseSettings):
    """Telegram bot configuration from environment variables."""

    model_config = SettingsConfigDict(
        env_file=str(ENV_FILE), env_file_encoding="utf-8", case_sensitive=False, extra="ignore"
    )

    bot_token: str = Field(..., alias="TELEGRAM_BOT_TOKEN")
    channel_id: str = Field(..., alias="TELEGRAM_CHANNEL_ID")
    dev_channel_id: str = Field(default="", alias="TELEGRAM_DEV_CHANNEL_ID")

    def get_active_channel_id(self, environment: str = "production") -> str:
        """Get the appropriate channel ID based on environment."""
        if environment == "development" and self.dev_channel_id:
            return self.dev_channel_id
        return self.channel_id


class PlatformConfig(BaseSettings):
    """Platform enable/disable configuration."""

    model_config = SettingsConfigDict(
        env_file=str(ENV_FILE), env_file_encoding="utf-8", case_sensitive=False, extra="ignore"
    )

    enable_twitter: bool = Field(default=False, alias="ENABLE_TWITTER")
    enable_telegram: bool = Field(default=True, alias="ENABLE_TELEGRAM")
    environment: str = Field(default="production", alias="ENVIRONMENT")

    def is_development(self) -> bool:
        """Check if running in development mode."""
        return self.environment.lower() == "development"

    def is_production(self) -> bool:
        """Check if running in production mode."""
        return self.environment.lower() == "production"


class DatabaseConfig(BaseSettings):
    """Database configuration for PostgreSQL."""

    model_config = SettingsConfigDict(
        env_file=str(ENV_FILE), env_file_encoding="utf-8", case_sensitive=False, extra="ignore"
    )

    # Full database URL (preferred)
    database_url: str = Field(
        default="postgresql://amazon_user:amazon_pass@localhost:5432/amazon_deals",
        alias="DATABASE_URL",
    )

    # Connection pooling
    db_pool_size: int = Field(default=10, alias="DB_POOL_SIZE")
    db_max_overflow: int = Field(default=20, alias="DB_MAX_OVERFLOW")

    # Query settings
    db_echo: bool = Field(default=False, alias="DB_ECHO")

    def get_database_url(self) -> str:
        """Get database URL."""
        return self.database_url


class MonitoringConfig(BaseSettings):
    """Monitoring and discovery configuration."""

    model_config = SettingsConfigDict(
        env_file=str(ENV_FILE), env_file_encoding="utf-8", case_sensitive=False, extra="ignore"
    )

    # Product discovery
    search_keywords: str = Field(..., alias="SEARCH_KEYWORDS")
    max_pages_per_keyword: int = Field(default=3, alias="MAX_PAGES_PER_KEYWORD")

    # Monitoring intervals (in seconds)
    discovery_interval: int = Field(default=3600, alias="DISCOVERY_INTERVAL_SECONDS")

    # Price limits
    max_price: float = Field(default=50000.0, alias="MAX_PRICE")

    def get_keywords_list(self) -> list[str]:
        """Parse keywords string into list."""
        return [kw.strip() for kw in self.search_keywords.split(",") if kw.strip()]



"""
New architecture keyword monitor - discovers products, monitors prices, detects deals.

This replaces the old deal_monitor_unified.py with a price history-based approach.
"""

import argparse
import logging
import time
from datetime import datetime, timezone

from sqlalchemy import func

from opportunity_core.models.database import Product
from opportunity_core.services.database_manager import init_database, get_db_manager
from opportunity_core.services.deal_detection import DealDetectionEngine, DetectedDeal
from opportunity_core.utils.load_config import MonitoringConfig
from opportunity_core.services.price_monitoring import PriceMonitoringService
from opportunity_core.services.product_discovery import ProductDiscoveryService
from opportunity_core.utils.authentication import Authentication, AuthenticationConfig
from opportunity_core.utils.load_config import (
    DatabaseConfig,
    DealConfig,
    PaapiConfig,
    PlatformConfig,
    TelegramConfig,
    TwitterConfig,
)
from opportunity_core.utils.telegram_notifier import DealInfo as TelegramDealInfo
from opportunity_core.utils.telegram_notifier import TelegramNotifier
from opportunity_core.utils.twitter_notifier import DealInfo as TwitterDealInfo
from opportunity_core.utils.twitter_notifier import TwitterNotifier


def setup_logging(environment: str = "production") -> logging.Logger:
    """Setup console logging only (database stores all data)."""
    console_handler = logging.StreamHandler()
    # Simplified format: [HH:MM:SS] MESSAGE
    formatter = logging.Formatter("[%(asctime)s] %(message)s", datefmt="%H:%M:%S")
    console_handler.setFormatter(formatter)

    logging.basicConfig(level=logging.INFO, handlers=[console_handler], force=True)
    return logging.getLogger(__name__)

#TODO Remove this line...
logger = logging.getLogger(__name__)


class PriceHistoryDealMonitor:
    """New architecture deal monitor using price history."""

    def __init__(self, service_name: str | None = None):
        """Initialize the deal monitor."""
        # Load configurations
        self.paapi_config = PaapiConfig()
        self.deal_config = DealConfig()
        self.platform_config = PlatformConfig()
        self.database_config = DatabaseConfig()
        self.service_name = service_name

        # Setup logging
        global logger
        logger = setup_logging(self.platform_config.environment)

        env_emoji = "🧪" if self.platform_config.is_development() else "🚀"
        env_name = "DEVELOPMENT" if self.platform_config.is_development() else "PRODUCTION"
        logger.info(f"{env_emoji} Environment: {env_name}")

        # Initialize database
        logger.info("🔌 Connecting to database...")
        init_database(self.database_config.get_database_url(), echo=False)
        self.db_manager = get_db_manager()
        logger.info("✅ Database connected")

        # Setup PA API authentication
        auth_config = AuthenticationConfig(
            access_key=self.paapi_config.access_key,
            secret_key=self.paapi_config.secret_key,
            partner_tag=self.paapi_config.partner_tag,
            host=self.paapi_config.host,
            region=self.paapi_config.region,
        )
        auth = Authentication(config=auth_config)
        api_client = auth.authenticate()

        # Initialize notifiers
        self.notifiers = []

        if self.platform_config.enable_telegram:
            telegram_config = TelegramConfig()
            self.telegram_notifier = TelegramNotifier(
                bot_token=telegram_config.bot_token,
                channel_id=telegram_config.get_active_channel_id(self.platform_config.environment),
            )
            self.notifiers.append(("Telegram", self.telegram_notifier))
            logger.info("✅ Telegram notifier enabled")

        if self.platform_config.enable_twitter:
            try:
                twitter_config = TwitterConfig()
                self.twitter_notifier = TwitterNotifier(
                    api_key=twitter_config.api_key,
                    api_secret=twitter_config.api_secret,
                    access_token=twitter_config.access_token,
                    access_token_secret=twitter_config.access_token_secret,
                    bearer_token=twitter_config.bearer_token,
                )
                self.notifiers.append(("Twitter", self.twitter_notifier))
                logger.info("✅ Twitter notifier enabled")
            except Exception as e:
                logger.error(f"❌ Failed to initialize Twitter notifier: {e}")
                logger.warning("⚠️  Continuing without Twitter support.")

        if not self.notifiers:
            logger.warning("⚠️  No notifiers enabled! Set ENABLE_TELEGRAM=true or ENABLE_TWITTER=true")

        # Initialize services (with database session)
        with self.db_manager.get_session() as session:
            self.discovery_service = ProductDiscoveryService(
                api_client=api_client,
                partner_tag=self.paapi_config.partner_tag,
                partner_type=self.paapi_config.partner_type,
                marketplace=self.paapi_config.marketplace,
                db_session=session,
            )

            self.price_monitor = PriceMonitoringService(
                api_client=api_client,
                partner_tag=self.paapi_config.partner_tag,
                partner_type=self.paapi_config.partner_type,
                marketplace=self.paapi_config.marketplace,
                db_session=session,
            )

            self.deal_detector = DealDetectionEngine(
                db_session=session,
                cooldown_hours=24,
            )

        # Load monitoring configuration from environment
        self.monitoring_config = MonitoringConfig()

        # Get keywords from config (externalized, no more hard-coded!)
        self.keywords_list = self.monitoring_config.get_keywords_list()
        logger.info(f"📋 Loaded {len(self.keywords_list)} keywords from config")

        logger.info("=" * 80)
        logger.info("✅ INITIALIZATION COMPLETE")
        logger.info("=" * 80)

    def run_discovery_phase(self):
        """Phase 1: Discover new products and build ASIN pool."""
        logger.info("\n" + "=" * 80)
        logger.info("🔍 PHASE 1: PRODUCT DISCOVERY")
        logger.info("=" * 80)

        with self.db_manager.get_session() as session:
            discovery = ProductDiscoveryService(
                api_client=self.discovery_service.api_client,
                partner_tag=self.paapi_config.partner_tag,
                partner_type=self.paapi_config.partner_type,
                marketplace=self.paapi_config.marketplace,
                db_session=session,
            )

            stats = discovery.discover_from_keyword_list(
                keywords_list=self.keywords_list,
                search_index="All",
                max_pages_per_keyword=3,  # Keep at 3 pages (relevance drops after that)
                category="general",
                use_variations=True,  # ENABLED
                variation_letters="abcdefghij",  # Increased to 10 variations (a-j)
            )

            logger.info(f"✅ Discovery phase complete: {stats['total_new']} new products")

    def run_price_monitoring_phase(self, segment_index: int | None = None, total_segments: int = 3):
        """Phase 2: Check prices for all active products."""
        logger.info("\n" + "=" * 80)
        logger.info("💰 PHASE 2: PRICE MONITORING")
        logger.info("=" * 80)

        with self.db_manager.get_session() as session:
            monitor = PriceMonitoringService(
                api_client=self.price_monitor.api_client,
                partner_tag=self.paapi_config.partner_tag,
                partner_type=self.paapi_config.partner_type,
                marketplace=self.paapi_config.marketplace,
                db_session=session,
            )

            # Initialize detector with current session for real-time alerts
            detector = DealDetectionEngine(db_session=session, cooldown_hours=24)

            def on_batch_complete(updated_asins: list[str]):
                if not updated_asins:
                    return

                # logger.info(f"⚡ Real-time detection for {len(updated_asins)} products...")
                deals = detector.detect_deals(target_asins=updated_asins)

                for deal in deals:
                    self._process_and_publish_deal(deal, detector)

            # Calculate segment based on current hour if not provided
            if segment_index is None:
                current_hour = datetime.now(timezone.utc).hour
                segment_index = current_hour % total_segments
                logger.info(f"📊 Auto-calculated segment: {segment_index + 1}/{total_segments} (Hour: {current_hour})")
            else:
                logger.info(f"📊 Manual segment: {segment_index + 1}/{total_segments}")

            stats = monitor.check_prices_for_active_products(
                batch_size=10,
                max_batches=None,
                on_batch_processed=on_batch_complete,
                segment_index=segment_index,
                total_segments=total_segments,
            )

            logger.info(
                f"✅ Price monitoring complete: {stats['products_checked']} checked, {stats['prices_updated']} updated"
            )

    def run_deal_detection_phase(self):
        """Phase 3: Detect deals using price history and publish."""
        logger.info("\n" + "=" * 80)
        logger.info("🎯 PHASE 3: DEAL DETECTION & PUBLISHING")
        logger.info("=" * 80)

        published_count = 0

        with self.db_manager.get_session() as session:
            detector = DealDetectionEngine(db_session=session, cooldown_hours=24)

            deals_generator = detector.detect_deals(limit=None)  # Check all products

            logger.info("Starting deal detection stream...")

            for deal in deals_generator:
                logger.info(f"\n📢 Publishing deal: [{deal.asin}] {deal.product.title[:50]}...")
                logger.info(f"   Type: {deal.deal_type.value}")
                logger.info(f"   Price: {deal.current_price} {deal.currency}")
                logger.info(f"   Reason: {deal.reason}")

                # Publish to all enabled platforms
                if self._process_and_publish_deal(deal, detector):
                    published_count += 1

                time.sleep(5)  # Delay between publications

        logger.info(f"\n✅ Deal detection complete: {published_count} deals published")

    def _publish_to_telegram(self, deal: DetectedDeal) -> str | None:
        """Publish deal to Telegram."""
        try:
            # Determine deal tag (FIRSAT vs FİYAT DÜŞTÜ)
            # If it's an official Amazon deal (indicated in reason or by deal type), use FIRSAT
            # Otherwise, if it's purely a historical drop, use FİYAT DÜŞTÜ
            deal_tag = "FIRSAT"
            if "Amazon Deal" in deal.reason:
                deal_tag = "FIRSAT"
            elif deal.deal_type.value == "ALL_TIME_LOW" or deal.deal_type.value == "DROP_24H":
                # If it's a historical drop without explicit Amazon Deal mention, check savings
                # If savings_percentage matches discount_percentage, it might be official
                # But safer to default to FİYAT DÜŞTÜ for historical comparisons unless we know otherwise
                deal_tag = "FİYAT DÜŞTÜ"

            deal_info = TelegramDealInfo(
                asin=deal.asin,
                title=deal.product.title or "Amazon Deal",
                current_price=deal.current_price,
                original_price=deal.previous_price or deal.current_price * 1.2,
                discount_percentage=int(deal.discount_percentage) if deal.discount_percentage else 0,
                currency=deal.currency,
                image_url=deal.product.image_url,
                detail_page_url=deal.product.url or f"https://{self.paapi_config.marketplace}/dp/{deal.asin}",
                rating=deal.product.rating,
                review_count=deal.product.review_count,
                is_prime_eligible=deal.product.is_prime_eligible,
                lowest_price_30d=deal.lowest_price_30d,
                lowest_price_90d=deal.lowest_price_90d,
                lowest_price_180d=deal.lowest_price_180d,
                first_discovered_at=deal.product.first_discovered_at,
                deal_tag=deal_tag,
            )
            result = self.telegram_notifier.post_deal(deal_info)
            return result.get("message_id") if result else None
        except Exception as e:
            logger.error(f"Telegram publish error: {e}")
            return None

    def _publish_to_twitter(self, deal: DetectedDeal) -> str | None:
        """Publish deal to Twitter."""
        try:
            deal_info = TwitterDealInfo(
                asin=deal.asin,
                title=deal.product.title or "Amazon Deal",
                current_price=deal.current_price,
                original_price=deal.previous_price or deal.current_price * 1.2,
                discount_percentage=int(deal.discount_percentage) if deal.discount_percentage else 15,
                currency=deal.currency,
                detail_page_url=deal.product.url or f"https://{self.paapi_config.marketplace}/dp/{deal.asin}",
                image_url=deal.product.image_url,
                lowest_price_30d=deal.lowest_price_30d,
                lowest_price_90d=deal.lowest_price_90d,
                lowest_price_180d=deal.lowest_price_180d,
                first_discovered_at=deal.product.first_discovered_at,
            )
            result = self.twitter_notifier.post_deal(deal_info)
            return result.get("tweet_id") if result else None
        except Exception as e:
            logger.error(f"Twitter publish error: {e}")
            return None

    def _process_and_publish_deal(self, deal: DetectedDeal, detector: DealDetectionEngine) -> bool:
        """Process a detected deal and publish to enabled platforms."""
        is_published = False

        logger.info(f"\n📢 Publishing deal: [{deal.asin}] {deal.product.title[:50]}...")
        logger.info(f"   Type: {deal.deal_type.value}")
        logger.info(f"   Price: {deal.current_price} {deal.currency}")
        logger.info(f"   Reason: {deal.reason}")

        for platform_name, notifier in self.notifiers:
            try:
                platform_key = platform_name.lower()

                if platform_name == "Telegram":
                    message_id = self._publish_to_telegram(deal)
                    if message_id:
                        logger.info(f"   ✅ Published to {platform_name}")
                        detector.record_alert(deal, platform=platform_key, message_id=message_id)
                        is_published = True

                elif platform_name == "Twitter":
                    # Check rate limit (15 tweets per 24h)
                    if detector.check_platform_rate_limit("twitter", limit=15, window_hours=24):
                        logger.warning(f"   ⚠️ Twitter rate limit reached (15/24h). Skipping {deal.asin}.")
                        continue

                    message_id = self._publish_to_twitter(deal)
                    if message_id:
                        logger.info(f"   ✅ Published to {platform_name}")
                        detector.record_alert(deal, platform=platform_key, message_id=message_id)
                        is_published = True

            except Exception as e:
                logger.error(f"   ❌ Error publishing to {platform_name}: {e}")

        return is_published

    def run_once(self):
        """Run a single complete monitoring cycle."""
        logger.info("\n" + "━" * 80)
        logger.info("🚀 CYCLE BAŞLADI")
        logger.info("━" * 80)

        start_time = time.time()

        # Store stats from each phase
        discovery_stats = {}
        monitoring_stats = {}
        detection_stats = {}

        # Phase 1: Discovery
        with self.db_manager.get_session() as session:
            discovery = ProductDiscoveryService(
                api_client=self.discovery_service.api_client,
                partner_tag=self.paapi_config.partner_tag,
                partner_type=self.paapi_config.partner_type,
                marketplace=self.paapi_config.marketplace,
                db_session=session,
            )
            discovery_stats = discovery.discover_from_keyword_list(
                keywords_list=self.keywords_list,
                search_index="All",
                max_pages_per_keyword=3,
            )

            # Run cleanup for inactive products (once per discovery cycle)
            deleted_count = discovery.cleanup_inactive_products(days_threshold=30)
            logger.info(f"🧹 Cleanup: {deleted_count} inactive products deactivated")

        time.sleep(10)

        # Phase 2: Price Monitoring
        with self.db_manager.get_session() as session:
            monitor = PriceMonitoringService(
                api_client=self.price_monitor.api_client,
                partner_tag=self.paapi_config.partner_tag,
                partner_type=self.paapi_config.partner_type,
                marketplace=self.paapi_config.marketplace,
                db_session=session,
            )
            monitoring_stats = monitor.check_prices_for_active_products(
                batch_size=10,
                max_batches=None,
            )
        time.sleep(10)

        # Phase 3: Deal Detection & Publishing
        published_count = 0
        with self.db_manager.get_session() as session:
            detector = DealDetectionEngine(db_session=session, cooldown_hours=24)
            deals_generator = detector.detect_deals(limit=None)

            for deal in deals_generator:
                for platform_name, notifier in self.notifiers:
                    try:
                        platform_key = platform_name.lower()

                        if platform_name == "Twitter":
                            # Rate Limit Check: Max 13 tweets per 24 hours
                            if detector.check_platform_rate_limit(platform="twitter", limit=13, window_hours=24):
                                logger.warning("🐦 Twitter rate limit reached (13/24h). Skipping tweet.")
                                continue

                        if platform_name == "Telegram":
                            from opportunity_core.utils.telegram_notifier import DealInfo as TelegramDealInfo

                            deal_info = TelegramDealInfo(
                                asin=deal.asin,
                                title=deal.product.title or "Amazon Deal",
                                current_price=deal.current_price,
                                original_price=deal.previous_price or deal.current_price * 1.2,
                                discount_percentage=int(deal.discount_percentage) if deal.discount_percentage else 0,
                                currency=deal.currency,
                                image_url=deal.product.image_url,
                                detail_page_url=deal.product.url
                                or f"https://{self.paapi_config.marketplace}/dp/{deal.asin}",
                                rating=deal.product.rating,
                                review_count=deal.product.review_count,
                                is_prime_eligible=deal.product.is_prime_eligible,
                                lowest_price_30d=deal.lowest_price_30d,
                                lowest_price_90d=deal.lowest_price_90d,
                                lowest_price_180d=deal.lowest_price_180d,
                                first_discovered_at=deal.product.first_discovered_at,
                            )
                            result = notifier.post_deal(deal_info)
                            if result and result.get("message_id"):
                                detector.record_alert(deal, platform=platform_key, message_id=result.get("message_id"))
                                published_count += 1

                    except Exception as e:
                        logger.error(f"   ❌ {platform_name} error: {e}")

                time.sleep(5)

        elapsed = time.time() - start_time

        # Calculate total database stats
        with self.db_manager.get_session() as session:
            from opportunity_core.models.database import Product

            total_products = session.query(Product).count()
            active_products = session.query(Product).filter(Product.is_active == True).count()

        # Print comprehensive summary
        logger.info("\n" + "━" * 80)
        logger.info("📊 CYCLE ÖZET")
        logger.info("━" * 80)
        logger.info(f"⏱️  Süre: {elapsed / 60:.1f} dakika ({elapsed:.0f} saniye)")
        logger.info("")
        logger.info(f"📦 FAZ 1 - ÜRÜN KEŞFİ:")
        logger.info(f"   • {discovery_stats.get('keywords_searched', 0)} keyword tarandı")
        logger.info(f"   • {discovery_stats.get('total_new', 0)} YENİ ürün keşfedildi")
        logger.info("")
        logger.info(f"💰 FAZ 2 - FİYAT TAKİBİ:")
        logger.info(f"   • {monitoring_stats.get('products_checked', 0)} ürün kontrol edildi")
        logger.info(f"   • {monitoring_stats.get('prices_updated', 0)} fiyat güncellendi")
        logger.info(f"   • {monitoring_stats.get('errors', 0)} hata")
        logger.info("")
        logger.info(f"🎯 FAZ 3 - FIRSAT TESPİTİ:")
        logger.info(f"   • {published_count} FIRSAT yayınlandı")
        logger.info("")
        logger.info(f"📈 VERİTABANI DURUMU:")
        logger.info(f"   • Toplam ürün: {total_products}")
        logger.info(f"   • Aktif ürün: {active_products}")
        logger.info("━" * 80)

    def _should_run_initial_discovery(self, lookback_hours: int) -> bool:
        """
        Check if we should run discovery on startup.
        If the system was active recently (last_seen_at < lookback_hours), skip initial discovery.
        """
        try:
            with self.db_manager.get_session() as session:
                # Get the most recent last_seen_at from Product table
                last_seen = session.query(func.max(Product.last_seen_at)).scalar()

                if not last_seen:
                    logger.info("🆕 No products found in database. Running initial discovery.")
                    return True

                # Ensure last_seen is timezone-aware (UTC)
                if last_seen.tzinfo is None:
                    last_seen = last_seen.replace(tzinfo=timezone.utc)

                now = datetime.now(timezone.utc)
                diff = now - last_seen
                hours_diff = diff.total_seconds() / 3600

                if hours_diff < lookback_hours:
                    logger.info(f"⏭️  System active recently ({hours_diff:.1f}h ago). Skipping initial discovery.")
                    return False

                logger.info(f"🔍 System inactive for {hours_diff:.1f}h. Running initial discovery.")
                return True

        except Exception as e:
            logger.error(f"⚠️ Error checking initial discovery status: {e}")
            return True  # Fail safe: run discovery

    def _should_run_initial_monitoring(self, lookback_minutes: int) -> tuple[bool, float]:
        """
        Check if we should run price monitoring on startup.

        Returns:
            tuple[bool, float]: (should_run, minutes_since_last_update)
        """
        # SKIP SMART WAIT IN DEVELOPMENT
        if self.platform_config.is_development():
            logger.info("🧪 Development mode: Skipping smart wait. Running monitoring immediately.")
            return True, 0.0

        try:
            with self.db_manager.get_session() as session:
                from opportunity_core.models.database import PriceStatistics

                # Get the most recent updated_at from PriceStatistics table
                last_update = session.query(func.max(PriceStatistics.updated_at)).scalar()

                if not last_update:
                    logger.info("🆕 No price history found. Running initial monitoring.")
                    return True, 0.0

                # Ensure last_update is timezone-aware (UTC)
                if last_update.tzinfo is None:
                    last_update = last_update.replace(tzinfo=timezone.utc)

                now = datetime.now(timezone.utc)
                diff = now - last_update
                minutes_diff = diff.total_seconds() / 60

                if minutes_diff < lookback_minutes:
                    logger.info(f"⏭️  Prices checked recently ({minutes_diff:.1f}m ago). Skipping initial monitoring.")
                    return False, minutes_diff

                logger.info(f"💰 Prices not checked for {minutes_diff:.1f}m. Running initial monitoring.")
                return True, minutes_diff

        except Exception as e:
            logger.error(f"⚠️ Error checking initial monitoring status: {e}")
            return True, 0.0  # Fail safe: run monitoring

    def run_continuous(self, interval_seconds: int = 3600, discovery_interval_cycles: int = 4):
        """
        Run continuous monitoring with intervals.

        Args:
            interval_seconds: Sleep time between cycles (default 3600s)
            discovery_interval_cycles: Run discovery every N cycles (default 4 = 12h if interval is 3h)
        """
        logger.info(f"🔄 Starting continuous monitoring (Interval: {interval_seconds}s)")
        logger.info(f"📅 Product Discovery will run every {discovery_interval_cycles} cycles")

        cycle_count = 0

        while True:
            try:
                # Schedule Logic (TRT is UTC+3)
                # 01:00 TRT = 22:00 UTC -> Discovery
                # 02:00 - 09:00 TRT = 23:00 - 06:00 UTC -> Sleep
                # 09:00 - 01:00 TRT = 06:00 - 22:00 UTC -> Monitoring

                now_utc = datetime.now(timezone.utc)
                hour_utc = now_utc.hour

                # 1. Nighttime Pause (02:00 - 09:00 TRT)
                if hour_utc >= 23 or hour_utc < 6:
                    logger.info(f"🌙 Nighttime pause (02:00-09:00 TRT). Sleeping for 1 hour...")
                    time.sleep(3600)
                    continue

                cycle_count += 1
                logger.info(f"\n🚀 STARTING CYCLE {cycle_count}")

                # 2. Discovery Phase (01:00 TRT / 22:00 UTC)
                # We run discovery only if it's the 22:00 UTC hour
                run_discovery = hour_utc == 22

                # Smart Startup Override:
                # If it's the very first cycle after restart, and we are NOT in the discovery hour,
                # check if we missed a discovery recently or if the DB is empty.
                if cycle_count == 1 and not run_discovery:
                    # Only run initial discovery if DB is empty or very stale (e.g. > 24h)
                    # We don't want to force discovery on every restart during the day
                    if self._should_run_initial_discovery(lookback_hours=24):
                        logger.info("🆕 Initial discovery triggered by smart startup.")
                        run_discovery = True

                if run_discovery:
                    logger.info("🔍 Phase 1: Product Discovery (Scheduled 01:00 TRT)")
                    self.run_discovery_phase()

                    # CRITICAL: Do NOT run monitoring after discovery at night.
                    # User requested: "Keşiften sonra herhangi bir process çalışmayacak"
                    # So we sleep until the next cycle (which will likely hit the night pause)
                    logger.info("😴 Discovery finished. Skipping monitoring to respect night schedule.")
                    logger.info(f"\n⏰ Waiting {interval_seconds}s until next cycle...\n")
                    time.sleep(interval_seconds)
                    continue

                # 3. Monitoring Phase (Segmented)
                # We split the workload into 3 segments to run every hour
                total_segments = 3
                current_segment = (cycle_count - 1) % total_segments

                # Smart Startup for Price Monitoring:
                if cycle_count == 1:
                    should_run_monitoring, minutes_since_last = self._should_run_initial_monitoring(lookback_minutes=60)
                else:
                    should_run_monitoring = True
                    minutes_since_last = 0.0

                if should_run_monitoring:
                    logger.info(f"💰 Phase 2: Price Monitoring (Segment {current_segment + 1}/{total_segments})")

                    updated_asins_in_segment = []

                    def on_batch_complete_segment(batch_asins: list[str]):
                        updated_asins_in_segment.extend(batch_asins)
                        with self.db_manager.get_session() as session:
                            detector = DealDetectionEngine(db_session=session, cooldown_hours=24)
                            deals = detector.detect_deals(target_asins=batch_asins)
                            for deal in deals:
                                self._process_and_publish_deal(deal, detector)

                    with self.db_manager.get_session() as session:
                        monitor = PriceMonitoringService(
                            api_client=self.price_monitor.api_client,
                            partner_tag=self.paapi_config.partner_tag,
                            partner_type=self.paapi_config.partner_type,
                            marketplace=self.paapi_config.marketplace,
                            db_session=session,
                        )

                        monitor.check_prices_for_active_products(
                            batch_size=10,
                            max_batches=None,
                            on_batch_processed=on_batch_complete_segment,
                            segment_index=current_segment,
                            total_segments=total_segments,
                        )

                    logger.info(f"🎯 Phase 3: Deal Detection (Targeted for {len(updated_asins_in_segment)} products)")
                    logger.info("✅ Deal detection handled in real-time during monitoring.")

                else:
                    remaining_seconds = max(0, interval_seconds - int(minutes_since_last * 60))
                    logger.info(f"⏭️  Skipping Price Monitoring (Run {minutes_since_last:.1f}m ago).")
                    logger.info(f"⏳ Smart Wait: Sleeping for {remaining_seconds}s to sync with schedule...")
                    time.sleep(remaining_seconds)
                    continue

                logger.info(f"\n⏰ Waiting {interval_seconds}s until next cycle...\n")
                time.sleep(interval_seconds)

            except KeyboardInterrupt:
                logger.info("\n⏹️  Shutting down gracefully...")
                break
            except Exception as e:
                logger.error(f"❌ Error in monitoring cycle: {e}", exc_info=True)
                logger.info(f"⏰ Retrying in {interval_seconds}s...")
                time.sleep(interval_seconds)

    def run_test_notification(self):
        """Run a single test notification with dummy data."""
        logger.info("\n" + "=" * 80)
        logger.info("🧪 RUNNING TEST NOTIFICATION")
        logger.info("=" * 80)

        from opportunity_core.services.deal_detection import DealType
        from opportunity_core.models.database import PriceStatistics

        # Create dummy product
        dummy_product = Product(
            asin="TEST123456",
            title="Philips Airfryer XXL Smart Sensing Fritöz (Test Ürünü)",
            url="https://www.amazon.com.tr/dp/TEST123456",
            image_url="https://m.media-amazon.com/images/I/61q6x-g5iJL._AC_SL1500_.jpg",
            rating=4.8,
            review_count=1500,
            is_prime_eligible=True,
            first_discovered_at=datetime.now(timezone.utc),
        )

        # Create dummy statistics
        dummy_stats = PriceStatistics(
            asin="TEST123456",
            current_price=3000.00,
            min_price_30d=4500.00,
            avg_price_30d=5000.00,
            min_price_all_time=3000.00,
        )

        # Create dummy deal
        dummy_deal = DetectedDeal(
            asin="TEST123456",
            deal_type=DealType.ALL_TIME_LOW,
            current_price=3000.00,
            previous_price=5000.00,
            discount_amount=2000.00,
            discount_percentage=40.0,
            currency="TRY",
            product=dummy_product,
            statistics=dummy_stats,
            confidence_score=1.0,
            reason="Test Notification (All Time Low)",
            lowest_price_30d=4500.00,
            lowest_price_90d=4800.00,
            lowest_price_180d=5000.00,
        )

        logger.info("Generated dummy deal data.")

        # Publish to Telegram
        if self.platform_config.enable_telegram:
            logger.info("Sending to Telegram...")
            msg_id = self._publish_to_telegram(dummy_deal)
            if msg_id:
                logger.info(f"✅ Telegram test successful! Message ID: {msg_id}")
            else:
                logger.error("❌ Telegram test failed.")

        # Publish to Twitter
        if self.platform_config.enable_twitter:
            logger.info("Sending to Twitter...")
            tweet_id = self._publish_to_twitter(dummy_deal)
            if tweet_id:
                logger.info(f"✅ Twitter test successful! Tweet ID: {tweet_id}")
            else:
                logger.error("❌ Twitter test failed.")

        logger.info("\nTest complete. Exiting.")


def main():
    """Main entry point."""
    parser = argparse.ArgumentParser(description="Amazon Deal Monitor - Price History Architecture")
    parser.add_argument(
        "--mode",
        choices=["once", "continuous", "discovery", "monitoring", "detection", "test_notification"],
        default="once",
        help="Run mode",
    )
    parser.add_argument(
        "--segment",
        type=int,
        help="Specific segment index to run (0-based)",
    )
    parser.add_argument(
        "--total-segments",
        type=int,
        default=3,
        help="Total number of segments (default: 3)",
    )

    args = parser.parse_args()

    monitor = PriceHistoryDealMonitor(service_name="keyword-monitor")

    if args.mode == "once":
        monitor.run_once()
    elif args.mode == "continuous":
        monitor.run_continuous(interval_seconds=args.interval)
    elif args.mode == "discovery":
        monitor.run_discovery_phase()
    elif args.mode == "monitoring":
        monitor.run_price_monitoring_phase(segment_index=args.segment, total_segments=args.total_segments)
    elif args.mode == "detection":
        monitor.run_deal_detection_phase()
    elif args.mode == "test_notification":
        monitor.run_test_notification()


if __name__ == "__main__":
    main()



from paapi5_python_sdk.api.default_api import DefaultApi
from pydantic import BaseModel


class AuthenticationConfig(BaseModel):
    """Configuration model for Amazon PA API authentication."""

    access_key: str
    secret_key: str
    partner_tag: str
    host: str
    region: str


class Authentication:
    """Handles authentication for Amazon Product Advertising API."""

    def __init__(self, config: AuthenticationConfig):
        """
        Initialize authentication with configuration.

        Args:
            config: Authentication configuration containing credentials and settings
        """
        self._api_client = None
        self.access_key = config.access_key
        self.secret_key = config.secret_key
        self.partner_tag = config.partner_tag
        self.host = config.host
        self.region = config.region

    def authenticate(self) -> DefaultApi:
        """
        Create and return an authenticated API client.

        Returns:
            DefaultApi: Authenticated API client

        Raises:
            ValueError: If authentication fails
        """
        if not all([self.access_key, self.secret_key, self.host, self.region]):
            raise ValueError("Missing required authentication credentials")
        try:
            self._api_client = DefaultApi(
                access_key=self.access_key,
                secret_key=self.secret_key,
                host=self.host,
                region=self.region,
            )
            print("Authentication successful.")
            return self._api_client
        except Exception as e:
            raise ValueError(f"Authentication failed: {str(e)}") from e

    def __repr__(self):
        return f"Authentication(partner_tag={self.partner_tag}, host={self.host}, region={self.region})"



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.offer_availability import OfferAvailability  # noqa: F401,E501
from paapi5_python_sdk.models.offer_condition import OfferCondition  # noqa: F401,E501
from paapi5_python_sdk.models.offer_delivery_info import OfferDeliveryInfo  # noqa: F401,E501
from paapi5_python_sdk.models.offer_loyalty_points import OfferLoyaltyPoints  # noqa: F401,E501
from paapi5_python_sdk.models.offer_merchant_info import OfferMerchantInfo  # noqa: F401,E501
from paapi5_python_sdk.models.offer_price import OfferPrice  # noqa: F401,E501
from paapi5_python_sdk.models.offer_program_eligibility import OfferProgramEligibility  # noqa: F401,E501
from paapi5_python_sdk.models.offer_promotion import OfferPromotion  # noqa: F401,E501


class OfferListing(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "availability": "OfferAvailability",
        "condition": "OfferCondition",
        "delivery_info": "OfferDeliveryInfo",
        "id": "str",
        "is_buy_box_winner": "bool",
        "loyalty_points": "OfferLoyaltyPoints",
        "merchant_info": "OfferMerchantInfo",
        "price": "OfferPrice",
        "program_eligibility": "OfferProgramEligibility",
        "promotions": "list[OfferPromotion]",
        "saving_basis": "OfferPrice",
        "violates_map": "bool",
    }

    attribute_map = {
        "availability": "Availability",
        "condition": "Condition",
        "delivery_info": "DeliveryInfo",
        "id": "Id",
        "is_buy_box_winner": "IsBuyBoxWinner",
        "loyalty_points": "LoyaltyPoints",
        "merchant_info": "MerchantInfo",
        "price": "Price",
        "program_eligibility": "ProgramEligibility",
        "promotions": "Promotions",
        "saving_basis": "SavingBasis",
        "violates_map": "ViolatesMAP",
    }

    def __init__(
        self,
        availability=None,
        condition=None,
        delivery_info=None,
        id=None,
        is_buy_box_winner=None,
        loyalty_points=None,
        merchant_info=None,
        price=None,
        program_eligibility=None,
        promotions=None,
        saving_basis=None,
        violates_map=None,
    ):  # noqa: E501
        """OfferListing - a model defined in Swagger"""  # noqa: E501

        self._availability = None
        self._condition = None
        self._delivery_info = None
        self._id = None
        self._is_buy_box_winner = None
        self._loyalty_points = None
        self._merchant_info = None
        self._price = None
        self._program_eligibility = None
        self._promotions = None
        self._saving_basis = None
        self._violates_map = None
        self.discriminator = None

        if availability is not None:
            self.availability = availability
        if condition is not None:
            self.condition = condition
        if delivery_info is not None:
            self.delivery_info = delivery_info
        if id is not None:
            self.id = id
        if is_buy_box_winner is not None:
            self.is_buy_box_winner = is_buy_box_winner
        if loyalty_points is not None:
            self.loyalty_points = loyalty_points
        if merchant_info is not None:
            self.merchant_info = merchant_info
        if price is not None:
            self.price = price
        if program_eligibility is not None:
            self.program_eligibility = program_eligibility
        if promotions is not None:
            self.promotions = promotions
        if saving_basis is not None:
            self.saving_basis = saving_basis
        if violates_map is not None:
            self.violates_map = violates_map

    @property
    def availability(self):
        """Gets the availability of this OfferListing.  # noqa: E501


        :return: The availability of this OfferListing.  # noqa: E501
        :rtype: OfferAvailability
        """
        return self._availability

    @availability.setter
    def availability(self, availability):
        """Sets the availability of this OfferListing.


        :param availability: The availability of this OfferListing.  # noqa: E501
        :type: OfferAvailability
        """

        self._availability = availability

    @property
    def condition(self):
        """Gets the condition of this OfferListing.  # noqa: E501


        :return: The condition of this OfferListing.  # noqa: E501
        :rtype: OfferCondition
        """
        return self._condition

    @condition.setter
    def condition(self, condition):
        """Sets the condition of this OfferListing.


        :param condition: The condition of this OfferListing.  # noqa: E501
        :type: OfferCondition
        """

        self._condition = condition

    @property
    def delivery_info(self):
        """Gets the delivery_info of this OfferListing.  # noqa: E501


        :return: The delivery_info of this OfferListing.  # noqa: E501
        :rtype: OfferDeliveryInfo
        """
        return self._delivery_info

    @delivery_info.setter
    def delivery_info(self, delivery_info):
        """Sets the delivery_info of this OfferListing.


        :param delivery_info: The delivery_info of this OfferListing.  # noqa: E501
        :type: OfferDeliveryInfo
        """

        self._delivery_info = delivery_info

    @property
    def id(self):
        """Gets the id of this OfferListing.  # noqa: E501


        :return: The id of this OfferListing.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OfferListing.


        :param id: The id of this OfferListing.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def is_buy_box_winner(self):
        """Gets the is_buy_box_winner of this OfferListing.  # noqa: E501


        :return: The is_buy_box_winner of this OfferListing.  # noqa: E501
        :rtype: bool
        """
        return self._is_buy_box_winner

    @is_buy_box_winner.setter
    def is_buy_box_winner(self, is_buy_box_winner):
        """Sets the is_buy_box_winner of this OfferListing.


        :param is_buy_box_winner: The is_buy_box_winner of this OfferListing.  # noqa: E501
        :type: bool
        """

        self._is_buy_box_winner = is_buy_box_winner

    @property
    def loyalty_points(self):
        """Gets the loyalty_points of this OfferListing.  # noqa: E501


        :return: The loyalty_points of this OfferListing.  # noqa: E501
        :rtype: OfferLoyaltyPoints
        """
        return self._loyalty_points

    @loyalty_points.setter
    def loyalty_points(self, loyalty_points):
        """Sets the loyalty_points of this OfferListing.


        :param loyalty_points: The loyalty_points of this OfferListing.  # noqa: E501
        :type: OfferLoyaltyPoints
        """

        self._loyalty_points = loyalty_points

    @property
    def merchant_info(self):
        """Gets the merchant_info of this OfferListing.  # noqa: E501


        :return: The merchant_info of this OfferListing.  # noqa: E501
        :rtype: OfferMerchantInfo
        """
        return self._merchant_info

    @merchant_info.setter
    def merchant_info(self, merchant_info):
        """Sets the merchant_info of this OfferListing.


        :param merchant_info: The merchant_info of this OfferListing.  # noqa: E501
        :type: OfferMerchantInfo
        """

        self._merchant_info = merchant_info

    @property
    def price(self):
        """Gets the price of this OfferListing.  # noqa: E501


        :return: The price of this OfferListing.  # noqa: E501
        :rtype: OfferPrice
        """
        return self._price

    @price.setter
    def price(self, price):
        """Sets the price of this OfferListing.


        :param price: The price of this OfferListing.  # noqa: E501
        :type: OfferPrice
        """

        self._price = price

    @property
    def program_eligibility(self):
        """Gets the program_eligibility of this OfferListing.  # noqa: E501


        :return: The program_eligibility of this OfferListing.  # noqa: E501
        :rtype: OfferProgramEligibility
        """
        return self._program_eligibility

    @program_eligibility.setter
    def program_eligibility(self, program_eligibility):
        """Sets the program_eligibility of this OfferListing.


        :param program_eligibility: The program_eligibility of this OfferListing.  # noqa: E501
        :type: OfferProgramEligibility
        """

        self._program_eligibility = program_eligibility

    @property
    def promotions(self):
        """Gets the promotions of this OfferListing.  # noqa: E501


        :return: The promotions of this OfferListing.  # noqa: E501
        :rtype: list[OfferPromotion]
        """
        return self._promotions

    @promotions.setter
    def promotions(self, promotions):
        """Sets the promotions of this OfferListing.


        :param promotions: The promotions of this OfferListing.  # noqa: E501
        :type: list[OfferPromotion]
        """

        self._promotions = promotions

    @property
    def saving_basis(self):
        """Gets the saving_basis of this OfferListing.  # noqa: E501


        :return: The saving_basis of this OfferListing.  # noqa: E501
        :rtype: OfferPrice
        """
        return self._saving_basis

    @saving_basis.setter
    def saving_basis(self, saving_basis):
        """Sets the saving_basis of this OfferListing.


        :param saving_basis: The saving_basis of this OfferListing.  # noqa: E501
        :type: OfferPrice
        """

        self._saving_basis = saving_basis

    @property
    def violates_map(self):
        """Gets the violates_map of this OfferListing.  # noqa: E501


        :return: The violates_map of this OfferListing.  # noqa: E501
        :rtype: bool
        """
        return self._violates_map

    @violates_map.setter
    def violates_map(self, violates_map):
        """Sets the violates_map of this OfferListing.


        :param violates_map: The violates_map of this OfferListing.  # noqa: E501
        :type: bool
        """

        self._violates_map = violates_map

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(OfferListing, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OfferListing):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

# flake8: noqa

from __future__ import absolute_import

"""
  Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  A copy of the License is located at

      http://www.apache.org/licenses/LICENSE-2.0

  or in the "license" file accompanying this file. This file is distributed
  on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
  express or implied. See the License for the specific language governing
  permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501

    OpenAPI spec version: 1.0.0

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

# import auth into sdk package
from paapi5_python_sdk.auth.sign_helper import AWSV4Auth



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.offer_condition_note import OfferConditionNote  # noqa: F401,E501
from paapi5_python_sdk.models.offer_sub_condition import OfferSubCondition  # noqa: F401,E501


class OfferCondition(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "display_value": "str",
        "label": "str",
        "locale": "str",
        "value": "str",
        "sub_condition": "OfferSubCondition",
        "condition_note": "OfferConditionNote",
    }

    attribute_map = {
        "display_value": "DisplayValue",
        "label": "Label",
        "locale": "Locale",
        "value": "Value",
        "sub_condition": "SubCondition",
        "condition_note": "ConditionNote",
    }

    def __init__(
        self,
        display_value=None,
        label=None,
        locale=None,
        value=None,
        sub_condition=None,
        condition_note=None,
    ):  # noqa: E501
        """OfferCondition - a model defined in Swagger"""  # noqa: E501

        self._display_value = None
        self._label = None
        self._locale = None
        self._value = None
        self._sub_condition = None
        self._condition_note = None
        self.discriminator = None

        if display_value is not None:
            self.display_value = display_value
        if label is not None:
            self.label = label
        if locale is not None:
            self.locale = locale
        if value is not None:
            self.value = value
        if sub_condition is not None:
            self.sub_condition = sub_condition
        if condition_note is not None:
            self.condition_note = condition_note

    @property
    def display_value(self):
        """Gets the display_value of this OfferCondition.  # noqa: E501


        :return: The display_value of this OfferCondition.  # noqa: E501
        :rtype: str
        """
        return self._display_value

    @display_value.setter
    def display_value(self, display_value):
        """Sets the display_value of this OfferCondition.


        :param display_value: The display_value of this OfferCondition.  # noqa: E501
        :type: str
        """

        self._display_value = display_value

    @property
    def label(self):
        """Gets the label of this OfferCondition.  # noqa: E501


        :return: The label of this OfferCondition.  # noqa: E501
        :rtype: str
        """
        return self._label

    @label.setter
    def label(self, label):
        """Sets the label of this OfferCondition.


        :param label: The label of this OfferCondition.  # noqa: E501
        :type: str
        """

        self._label = label

    @property
    def locale(self):
        """Gets the locale of this OfferCondition.  # noqa: E501


        :return: The locale of this OfferCondition.  # noqa: E501
        :rtype: str
        """
        return self._locale

    @locale.setter
    def locale(self, locale):
        """Sets the locale of this OfferCondition.


        :param locale: The locale of this OfferCondition.  # noqa: E501
        :type: str
        """

        self._locale = locale

    @property
    def value(self):
        """Gets the value of this OfferCondition.  # noqa: E501


        :return: The value of this OfferCondition.  # noqa: E501
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this OfferCondition.


        :param value: The value of this OfferCondition.  # noqa: E501
        :type: str
        """

        self._value = value

    @property
    def sub_condition(self):
        """Gets the sub_condition of this OfferCondition.  # noqa: E501


        :return: The sub_condition of this OfferCondition.  # noqa: E501
        :rtype: OfferSubCondition
        """
        return self._sub_condition

    @sub_condition.setter
    def sub_condition(self, sub_condition):
        """Sets the sub_condition of this OfferCondition.


        :param sub_condition: The sub_condition of this OfferCondition.  # noqa: E501
        :type: OfferSubCondition
        """

        self._sub_condition = sub_condition

    @property
    def condition_note(self):
        """Gets the condition_note of this OfferCondition.  # noqa: E501


        :return: The condition_note of this OfferCondition.  # noqa: E501
        :rtype: OfferConditionNote
        """
        return self._condition_note

    @condition_note.setter
    def condition_note(self, condition_note):
        """Sets the condition_note of this OfferCondition.


        :param condition_note: The condition_note of this OfferCondition.  # noqa: E501
        :type: OfferConditionNote
        """

        self._condition_note = condition_note

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(OfferCondition, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OfferCondition):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Database models for Amazon Deal Tracker."""

from datetime import UTC, datetime

from sqlalchemy import (
    Boolean,
    Column,
    DateTime,
    Float,
    ForeignKey,
    Index,
    Integer,
    Numeric,
    String,
    Text,
    UniqueConstraint,
)
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship

Base = declarative_base()


class Seller(Base):
    """Stores seller information to normalize seller data."""

    __tablename__ = "sellers"

    id = Column(Integer, primary_key=True, autoincrement=True)
    seller_name = Column(String(255), unique=True, nullable=False, index=True)
    is_amazon = Column(Boolean, default=False, nullable=False, index=True)
    first_seen_at = Column(DateTime, default=lambda: datetime.now(UTC), nullable=False)
    last_seen_at = Column(
        DateTime, default=lambda: datetime.now(UTC), onupdate=lambda: datetime.now(UTC), nullable=False
    )
    total_products = Column(Integer, default=0, nullable=False)

    # Relationships
    products = relationship("Product", back_populates="seller")
    price_records = relationship("PriceHistory", back_populates="seller")

    def __repr__(self):
        return f"<Seller(id={self.id}, name='{self.seller_name}', is_amazon={self.is_amazon})>"


class Product(Base):
    """Stores discovered ASINs - the product pool we track."""

    __tablename__ = "products"

    asin = Column(String(10), primary_key=True, index=True)
    title = Column(Text, nullable=True)
    url = Column(Text, nullable=True)
    image_url = Column(Text, nullable=True)
    category = Column(String(100), nullable=True, index=True)

    # Discovery metadata
    first_discovered_at = Column(DateTime, default=lambda: datetime.now(UTC), nullable=False)
    last_seen_at = Column(
        DateTime, default=lambda: datetime.now(UTC), onupdate=lambda: datetime.now(UTC), nullable=False
    )
    discovery_keywords = Column(Text, nullable=True)  # Keywords that found this product

    # Tracking flags
    is_active = Column(Boolean, default=True, nullable=False, index=True)  # Should we track this?
    check_priority = Column(Integer, default=1, nullable=False)  # 1=high, 2=medium, 3=low
    next_check_at = Column(
        DateTime, default=lambda: datetime.now(UTC), nullable=False, index=True
    )  # Adaptive scheduling

    # Product metadata (cached from PA API)
    rating = Column(Float, nullable=True)
    review_count = Column(Integer, nullable=True)
    is_prime_eligible = Column(Boolean, default=False)
    seller_id = Column(Integer, ForeignKey("sellers.id", ondelete="SET NULL"), nullable=True, index=True)

    # Relationships
    price_history = relationship("PriceHistory", back_populates="product", cascade="all, delete-orphan")
    alerts = relationship("AlertSent", back_populates="product", cascade="all, delete-orphan")
    seller = relationship("Seller", back_populates="products")

    # Indexes for common queries
    __table_args__ = (
        Index("ix_products_active_priority", "is_active", "check_priority"),
        Index("ix_products_last_seen", "last_seen_at"),
        Index("ix_products_seller", "seller_id"),
        Index("ix_products_category_active", "category", "is_active"),
    )

    def __repr__(self):
        return f"<Product(asin='{self.asin}', title='{self.title[:50] if self.title else None}')>"


class PriceHistory(Base):
    """Time series price data - tracks price changes over time."""

    __tablename__ = "price_history"

    id = Column(Integer, primary_key=True, autoincrement=True)
    asin = Column(String(10), ForeignKey("products.asin", ondelete="CASCADE"), nullable=False, index=True)

    # Price data
    price_amount = Column(Numeric(10, 2), nullable=False)
    currency = Column(String(3), default="TRY", nullable=False)

    # Original/list price (if available from Amazon)
    list_price = Column(Numeric(10, 2), nullable=True)
    savings_amount = Column(Numeric(10, 2), nullable=True)
    savings_percentage = Column(Integer, nullable=True)

    # Seller information (keep for historical data)
    seller_name = Column(String(255), nullable=True)
    seller_id = Column(Integer, ForeignKey("sellers.id", ondelete="SET NULL"), nullable=True, index=True)
    is_amazon_seller = Column(Boolean, default=False)
    is_prime_eligible = Column(Boolean, default=False)
    is_free_shipping = Column(Boolean, default=False)
    availability_message = Column(String(255), nullable=True)

    # Timestamp
    captured_at = Column(DateTime, default=lambda: datetime.now(UTC), nullable=False)

    # Relationships
    product = relationship("Product", back_populates="price_history")
    seller = relationship("Seller", back_populates="price_records")

    # Indexes for time-series queries
    __table_args__ = (
        Index("ix_price_history_asin_captured", "asin", "captured_at"),
        Index("ix_price_history_captured_at", "captured_at"),
        Index("ix_price_history_seller", "seller_id"),
    )

    def __repr__(self):
        return f"<PriceHistory(asin='{self.asin}', price={self.price_amount}, captured_at='{self.captured_at}')>"


class AlertSent(Base):
    """Tracks sent alerts to prevent duplicates and spam."""

    __tablename__ = "alerts_sent"

    id = Column(Integer, primary_key=True, autoincrement=True)
    asin = Column(String(10), ForeignKey("products.asin", ondelete="CASCADE"), nullable=False, index=True)

    # Alert metadata
    alert_type = Column(String(50), nullable=False, index=True)  # e.g., DROP_24H, NEW_LOW_30D, FIRST_TIME_DEAL
    price_amount = Column(Numeric(10, 2), nullable=False)
    previous_price = Column(Numeric(10, 2), nullable=True)
    discount_percentage = Column(Integer, nullable=True)

    # Publishing details
    sent_at = Column(DateTime, default=lambda: datetime.now(UTC), nullable=False, index=True)
    platform = Column(String(20), nullable=False)  # telegram, twitter
    message_id = Column(String(100), nullable=True)  # Platform-specific message ID

    # Cooldown management
    cooldown_until = Column(DateTime, nullable=True, index=True)

    # Relationship
    product = relationship("Product", back_populates="alerts")

    # Prevent duplicate alerts
    __table_args__ = (
        Index("ix_alerts_asin_sent_at", "asin", "sent_at"),
        Index("ix_alerts_asin_cooldown", "asin", "cooldown_until"),
        UniqueConstraint("asin", "alert_type", "sent_at", name="uq_alert_asin_type_time"),
    )

    def __repr__(self):
        return f"<AlertSent(asin='{self.asin}', type='{self.alert_type}', price={self.price_amount})>"


class PriceStatistics(Base):
    """Materialized view / cache of price statistics for fast queries."""

    __tablename__ = "price_statistics"

    asin = Column(String(10), ForeignKey("products.asin", ondelete="CASCADE"), primary_key=True)

    # Current price
    current_price = Column(Numeric(10, 2), nullable=True)
    current_price_updated_at = Column(DateTime, nullable=True)

    # Historical minimums
    min_price_24h = Column(Numeric(10, 2), nullable=True)
    min_price_7d = Column(Numeric(10, 2), nullable=True)
    min_price_30d = Column(Numeric(10, 2), nullable=True)
    min_price_all_time = Column(Numeric(10, 2), nullable=True)

    # Historical maximums
    max_price_24h = Column(Numeric(10, 2), nullable=True)
    max_price_7d = Column(Numeric(10, 2), nullable=True)
    max_price_30d = Column(Numeric(10, 2), nullable=True)

    # Averages
    avg_price_7d = Column(Numeric(10, 2), nullable=True)
    avg_price_30d = Column(Numeric(10, 2), nullable=True)

    # Price change indicators
    price_change_24h = Column(Numeric(10, 2), nullable=True)  # Absolute change
    price_change_24h_pct = Column(Float, nullable=True)  # Percentage change

    # Volatility
    price_volatility_30d = Column(Float, nullable=True)  # Standard deviation

    # Last update
    updated_at = Column(DateTime, default=lambda: datetime.now(UTC), onupdate=lambda: datetime.now(UTC), nullable=False)

    def __repr__(self):
        return f"<PriceStatistics(asin='{self.asin}', current={self.current_price})>"



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.price import Price  # noqa: F401,E501
from paapi5_python_sdk.models.variation_dimension import VariationDimension  # noqa: F401,E501


class VariationSummary(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "page_count": "int",
        "price": "Price",
        "variation_count": "int",
        "variation_dimensions": "list[VariationDimension]",
    }

    attribute_map = {
        "page_count": "PageCount",
        "price": "Price",
        "variation_count": "VariationCount",
        "variation_dimensions": "VariationDimensions",
    }

    def __init__(
        self,
        page_count=None,
        price=None,
        variation_count=None,
        variation_dimensions=None,
    ):  # noqa: E501
        """VariationSummary - a model defined in Swagger"""  # noqa: E501

        self._page_count = None
        self._price = None
        self._variation_count = None
        self._variation_dimensions = None
        self.discriminator = None

        if page_count is not None:
            self.page_count = page_count
        if price is not None:
            self.price = price
        if variation_count is not None:
            self.variation_count = variation_count
        if variation_dimensions is not None:
            self.variation_dimensions = variation_dimensions

    @property
    def page_count(self):
        """Gets the page_count of this VariationSummary.  # noqa: E501


        :return: The page_count of this VariationSummary.  # noqa: E501
        :rtype: int
        """
        return self._page_count

    @page_count.setter
    def page_count(self, page_count):
        """Sets the page_count of this VariationSummary.


        :param page_count: The page_count of this VariationSummary.  # noqa: E501
        :type: int
        """

        self._page_count = page_count

    @property
    def price(self):
        """Gets the price of this VariationSummary.  # noqa: E501


        :return: The price of this VariationSummary.  # noqa: E501
        :rtype: Price
        """
        return self._price

    @price.setter
    def price(self, price):
        """Sets the price of this VariationSummary.


        :param price: The price of this VariationSummary.  # noqa: E501
        :type: Price
        """

        self._price = price

    @property
    def variation_count(self):
        """Gets the variation_count of this VariationSummary.  # noqa: E501


        :return: The variation_count of this VariationSummary.  # noqa: E501
        :rtype: int
        """
        return self._variation_count

    @variation_count.setter
    def variation_count(self, variation_count):
        """Sets the variation_count of this VariationSummary.


        :param variation_count: The variation_count of this VariationSummary.  # noqa: E501
        :type: int
        """

        self._variation_count = variation_count

    @property
    def variation_dimensions(self):
        """Gets the variation_dimensions of this VariationSummary.  # noqa: E501


        :return: The variation_dimensions of this VariationSummary.  # noqa: E501
        :rtype: list[VariationDimension]
        """
        return self._variation_dimensions

    @variation_dimensions.setter
    def variation_dimensions(self, variation_dimensions):
        """Sets the variation_dimensions of this VariationSummary.


        :param variation_dimensions: The variation_dimensions of this VariationSummary.  # noqa: E501
        :type: list[VariationDimension]
        """

        self._variation_dimensions = variation_dimensions

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(VariationSummary, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, VariationSummary):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""
Comment Generator - Adds human-like context to deal notifications.
"""

import random
from typing import Optional

# Circular import prevention: we only need the data structure, but we can't import DealInfo
# if it's in a file that imports this one. So we'll use duck typing or a protocol if needed,
# or just accept 'Any' and document the expected fields.
# For now, we'll assume the input object has the necessary attributes.


class CommentGenerator:
    """Generates human-like comments for deals."""

    # Category Keywords Mapping
    CATEGORY_KEYWORDS = {
        "tech": [
            "laptop",
            "bilgisayar",
            "telefon",
            "kulaklık",
            "mouse",
            "klavye",
            "tablet",
            "monitor",
            "ekran",
            "usb",
            "şarj",
        ],
        "home": [
            "kahve",
            "tava",
            "tencere",
            "ütü",
            "süpürge",
            "yastık",
            "yorgan",
            "nevresim",
            "banyo",
            "mutfak",
            "airfryer",
        ],
        "baby": ["bebek", "bez", "mama", "oyuncak", "puset", "oto koltuğu", "biberon", "emzik"],
        "fashion": ["ayakkabı", "bot", "mont", "ceket", "tişört", "pantolon", "çanta", "saat", "gözlük"],
        "book": ["kitap", "roman", "edebiyat", "hikaye"],
        "pet": ["kedi", "köpek", "mama", "kum"],
        "gaming": ["oyun", "konsol", "ps5", "xbox", "nintendo", "gamepad"],
    }

    # Comment Templates
    COMMENTS = {
        "all_time_low": [
            "Tarihi dip fiyat! Daha ucuzu görülmedi.",
            "Bu fiyata daha önce düşmemişti, tam alım fırsatı!",
            "Rekor düşük fiyat! İhtiyacınız varsa kaçırmayın.",
            "Efsane fiyat! Bu seviyeyi bir daha zor görürüz.",
        ],
        "huge_discount": [  # > 50%
            "Yarı fiyatına düşmüş, stoklar dayanmayabilir!",
            "İnanılmaz bir indirim oranı, hemen değerlendirin.",
            "Bedavadan biraz pahalı! Bu fırsat kaçmaz.",
            "Koşun! Fiyat hatası gibi duruyor.",
        ],
        "big_discount": [  # > 30%
            "Çok ciddi bir indirim oranı, kaçırmayın.",
            "Fiyatı gayet makul seviyeye inmiş.",
            "Bütçe dostu bir fırsat yakaladık.",
            "Gözden kaçırılmayacak bir indirim.",
        ],
        "tech": [
            "Teknoloji tutkunları için harika fırsat.",
            "Cihazını yenilemek isteyenler bakabilir.",
            "Performans arayanlar için güzel seçenek.",
            "Dijital ihtiyaçlarınız için uygun fiyat.",
        ],
        "home": [
            "Evinizin ihtiyacı olabilir, fiyatı düşmüşken bakın.",
            "Ev ekonomisine katkı sağlayacak bir indirim.",
            "Mutfakta işinizi kolaylaştıracak bir ürün.",
            "Evinizi güzelleştirmek için fırsat.",
        ],
        "baby": [
            "Bebek ihtiyaçları için stok yapmalık fiyat.",
            "Anne-baba adayları bu indirimi kaçırmasın.",
            "Minikler için güzel bir sürpriz olabilir.",
        ],
        "fashion": [
            "Gardırobunuzu yenilemek için güzel zaman.",
            "Şıklığı ucuza getirmek isteyenlere.",
            "Sezonun trendi, fiyatı da düşmüş.",
        ],
        "gaming": [
            "Oyuncular toplanın! Güzel bir indirim var.",
            "Setup'ını güçlendirmek isteyenler kaçırmasın.",
            "Oyun keyfini artıracak bir fırsat.",
        ],
        "pet": [
            "Patili dostlarımız için mama/kum stoğu fırsatı.",
            "Kediniz/köpeğiniz buna bayılacak.",
        ],
        "generic": [
            "Fiyat/performans oranı dikkat çekici.",
            "Değerlendirilmesi gereken bir fırsat.",
            "Fiyat takibimize takılan güzel bir ürün.",
            "Kullanıcı yorumları genelde olumlu olan bir ürün.",
            "Etiket fiyatına göre avantajlı duruyor.",
        ],
    }

    @classmethod
    def generate_comment(cls, deal) -> str:
        """
        Generate a context-aware comment for the deal.

        Args:
            deal: DealInfo object (duck-typed)

        Returns:
            str: A short, human-like comment.
        """
        candidates = []

        # 1. Priority: All Time Low
        # We check if lowest_price_all_time logic exists or infer from deal_type if available
        # Since DealInfo might not have deal_type directly exposed or it might be a string,
        # we'll rely on the 'deal_tag' or price comparison if possible.
        # But DealInfo has 'lowest_price_180d' etc.
        # Let's assume if discount > 10% and it's close to lowest_price_180d, it's good.

        # Better: Check deal_tag or reason if available, but DealInfo is simple.
        # Let's use the data we have.

        is_all_time_low = False
        if hasattr(deal, "deal_tag") and deal.deal_tag == "FİYAT DÜŞTÜ":
            # If it's a price drop alert, check if it's near 180d low
            if deal.lowest_price_180d and deal.current_price <= deal.lowest_price_180d:
                is_all_time_low = True

        if is_all_time_low:
            candidates.extend(cls.COMMENTS["all_time_low"])

        # 2. Priority: Discount Depth
        if deal.discount_percentage >= 50:
            candidates.extend(cls.COMMENTS["huge_discount"])
        elif deal.discount_percentage >= 30:
            candidates.extend(cls.COMMENTS["big_discount"])

        # 3. Priority: Category Context
        title_lower = deal.title.lower()
        category_match = None

        for cat, keywords in cls.CATEGORY_KEYWORDS.items():
            if any(k in title_lower for k in keywords):
                category_match = cat
                break

        if category_match:
            candidates.extend(cls.COMMENTS[category_match])

        # 4. Fallback
        if not candidates:
            candidates.extend(cls.COMMENTS["generic"])

        # Select one random comment from the candidates
        # We might want to weight them? For now, random is fine.
        # If we have multiple types (e.g. Tech + Huge Discount), we mix them in the pool
        # and pick one. This gives variety.

        selected_comment = random.choice(candidates)

        return selected_comment



"""Price monitoring service - tracks price changes over time."""

import logging
import time
from datetime import UTC, datetime, timedelta
from typing import Any, Callable, Optional

from sqlalchemy import and_, func
from sqlalchemy.orm import Session

from opportunity_core.models.database import PriceHistory, PriceStatistics, Product
from opportunity_core.utils.api_helper import call_with_backoff
from paapi5_python_sdk.api.default_api import DefaultApi
from paapi5_python_sdk.models import GetItemsRequest, GetItemsResource
from paapi5_python_sdk.rest import ApiException

logger = logging.getLogger(__name__)


class PriceMonitoringService:
    """Monitors and records price changes for tracked products."""

    def __init__(
        self,
        api_client: DefaultApi,
        partner_tag: str,
        partner_type: str,
        marketplace: str,
        db_session: Session,
    ):
        """
        Initialize price monitoring service.

        Args:
            api_client: Authenticated PA API client
            partner_tag: Amazon partner tag
            partner_type: Partner type
            marketplace: Amazon marketplace domain
            db_session: Database session
        """
        self.api_client = api_client
        self.partner_tag = partner_tag
        self.partner_type = partner_type
        self.marketplace = marketplace
        self.db_session = db_session

    def check_prices_for_active_products(
        self,
        batch_size: int = 10,
        max_batches: Optional[int] = None,
        on_batch_processed: Optional[Callable[[list[str]], None]] = None,
        segment_index: int = 0,
        total_segments: int = 1,
    ) -> dict[str, int]:
        """
        Check prices for all active products in batches.

        Args:
            batch_size: Number of ASINs to check per API call (max 10)
            max_batches: Maximum number of batches to process (None = all)
            on_batch_processed: Callback function called with list of updated ASINs after each batch
            segment_index: Index of the current segment (0 to total_segments-1)
            total_segments: Total number of segments to split the workload into

        Returns:
            Statistics: {products_checked, prices_updated, errors}
        """
        batch_size = min(batch_size, 10)  # PA API limit
        products_checked = 0
        prices_updated = 0
        errors = 0

        # SMART MONITORING: Select products due for check
        now_utc = datetime.now(UTC)

        # 1. Base Query: Active products only
        query = (
            self.db_session.query(Product)
            .filter(Product.is_active == True)
            .filter(Product.next_check_at <= datetime.now(UTC))
            .order_by(Product.check_priority, Product.next_check_at)
        )

        # 4. Global Limit for Safety
        # Optimization: Decreased from 3500 to 2500 to reduce hourly load
        # Calculation: 2500 products/hr / 10 batch = 250 req/hr
        GLOBAL_LIMIT = 2500

        logger.info(f"📊 Selecting max {GLOBAL_LIMIT} products due for check (Priority-based)...")
        active_products = query.limit(GLOBAL_LIMIT).all()

        if not active_products:
            logger.info("✅ No products due for check. Smart monitoring allows skipping this cycle.")
            return {
                "products_checked": 0,
                "prices_updated": 0,
                "errors": 0,
            }

        total_products = len(active_products)

        # Process in batches
        for i in range(0, total_products, batch_size):
            if max_batches and i // batch_size >= max_batches:
                logger.info(f"⏸️  Reached max batches limit ({max_batches})")
                break

            batch = active_products[i : i + batch_size]
            asins = [p.asin for p in batch]

            logger.info(f"  Batch {i // batch_size + 1}: Checking {len(asins)} products...")

            try:
                results = self._check_prices_batch(asins)
                products_checked += len(asins)
                prices_updated += len([r for r in results if r])

                # Trigger callback for real-time detection
                if on_batch_processed:
                    try:
                        on_batch_processed(asins)
                    except Exception as cb_err:
                        logger.error(f"Error in batch callback: {cb_err}")

            except Exception as e:
                logger.error(f"Error checking batch: {e}")
                errors += len(asins)

            # Rate limiting between batches (Optimized: 2.0s is safe for GetItems with 10 items)
            time.sleep(2.0)

        logger.info(f"✅ Price check complete: {products_checked} checked, {prices_updated} updated, {errors} errors")

        return {
            "products_checked": products_checked,
            "prices_updated": prices_updated,
            "errors": errors,
        }

    def _check_prices_batch(self, asins: list[str]) -> list[Optional[PriceHistory]]:
        """
        Check prices for a batch of ASINs using GetItems API.

        Args:
            asins: List of ASINs to check (max 10)

        Returns:
            List of PriceHistory records created (None for failures)
        """
        results = []

        try:
            # Request comprehensive price and seller info
            request = GetItemsRequest(
                partner_tag=self.partner_tag,
                partner_type=self.partner_type,
                marketplace=self.marketplace,
                item_ids=asins,
                resources=[
                    GetItemsResource.OFFERS_LISTINGS_PRICE,
                    GetItemsResource.OFFERS_LISTINGS_SAVINGBASIS,
                    GetItemsResource.OFFERS_LISTINGS_MERCHANTINFO,
                    GetItemsResource.OFFERS_LISTINGS_AVAILABILITY_MESSAGE,
                    GetItemsResource.OFFERS_LISTINGS_DELIVERYINFO_ISPRIMEELIGIBLE,
                    GetItemsResource.OFFERS_LISTINGS_DELIVERYINFO_ISFREESHIPPINGELIGIBLE,
                    GetItemsResource.ITEMINFO_TITLE,
                ],
            )

            # Use backoff for API call
            response = call_with_backoff(
                self.api_client.get_items,
                request,
                max_retries=10,  # Increased from 3 to 10
                initial_delay=5.0,
                max_delay=120.0,  # Allow waiting up to 2 minutes
                backoff_factor=2.0,
            )

            if response and hasattr(response, "items_result") and response.items_result:
                if hasattr(response.items_result, "items") and response.items_result.items:
                    for item in response.items_result.items:
                        price_record = self._extract_and_store_price(item)
                        results.append(price_record)

            # Handle errors for specific ASINs
            if response and hasattr(response, "errors") and response.errors:
                for error in response.errors:
                    logger.warning(f"  Error for ASIN: {error}")

                    # Check for InvalidParameterValue error (invalid ASIN)
                    if hasattr(error, "code") and error.code == "InvalidParameterValue":
                        # Try to extract ASIN from message
                        # Message format: "The ItemIds B0998F82WC provided in the request is invalid."
                        import re

                        msg = error.message if hasattr(error, "message") else str(error)
                        match = re.search(r"ItemIds\s+([A-Z0-9]{10})", msg)

                        if match:
                            invalid_asin = match.group(1)
                            logger.warning(f"🚫 Invalid ASIN detected: {invalid_asin}. Deleting product.")

                            # Delete product from database
                            try:
                                product = self.db_session.query(Product).filter(Product.asin == invalid_asin).first()
                                if product:
                                    self.db_session.delete(product)
                                    self.db_session.commit()
                                    logger.info(f"✅ Product {invalid_asin} deleted successfully.")
                            except Exception as db_err:
                                logger.error(f"Failed to delete invalid product {invalid_asin}: {db_err}")
                                self.db_session.rollback()

                    results.append(None)

        except ApiException as e:
            logger.error(f"PA API error in batch price check: {e}")
            results = [None] * len(asins)
        except Exception as e:
            logger.error(f"Unexpected error in batch price check: {e}")
            results = [None] * len(asins)

        return results

    def cleanup_inactive_products(self, days_threshold: int = 30) -> int:
        """
        Deactivate products that have been out of stock or have no price updates for N days.

        Args:
            days_threshold: Number of days to look back (default 30)

        Returns:
            Number of products deactivated
        """
        logger.info(f"🧹 Starting cleanup of inactive products (Threshold: {days_threshold} days)...")

        cutoff_date = datetime.now(UTC) - timedelta(days=days_threshold)
        deactivated_count = 0

        try:
            # 1. Get all currently active products
            active_products = self.db_session.query(Product).filter(Product.is_active == True).all()

            for product in active_products:
                # Check if we have ANY price history in the last N days
                recent_history = (
                    self.db_session.query(PriceHistory)
                    .filter(and_(PriceHistory.asin == product.asin, PriceHistory.captured_at >= cutoff_date))
                    .count()
                )

                # If no history at all in 30 days, deactivate
                if recent_history == 0:
                    logger.info(f"  [{product.asin}] No price history in {days_threshold} days. Deactivating.")
                    product.is_active = False
                    deactivated_count += 1
                    continue

                # If there is history, check if it's ALL "out of stock" (price=0 or None)
                # But our PriceHistory only stores valid prices usually.
                # If we store 0 for out of stock, we check that.
                # Currently _extract_and_store_price returns None if no record is created.
                # So 'recent_history == 0' effectively covers "Out of Stock" if we don't record OOS.

                # However, if we updated last_seen_at but didn't add price history, that means OOS.
                # Let's check last_seen_at vs cutoff
                if product.last_seen_at and product.last_seen_at < cutoff_date:
                    logger.info(f"  [{product.asin}] Last seen > {days_threshold} days ago. Deactivating.")
                    product.is_active = False
                    deactivated_count += 1

            self.db_session.commit()
            logger.info(f"✅ Cleanup complete. Deactivated {deactivated_count} products.")
            return deactivated_count

        except Exception as e:
            logger.error(f"Error during product cleanup: {e}")
            self.db_session.rollback()
            return 0

    def _extract_and_store_price(self, item: Any) -> Optional[PriceHistory]:
        """
        Extract price data from API response and store in database.

        Args:
            item: PA API GetItems response item

        Returns:
            PriceHistory record if successful, None otherwise
        """
        try:
            # Extract ASIN
            asin = item.asin if hasattr(item, "asin") else None
            if not asin:
                return None

            # Extract price from offers
            current_price = None
            currency = "TRY"
            list_price = None
            savings_amount = None
            savings_percentage = None
            seller_name = None
            is_amazon_seller = False
            is_prime_eligible = False
            is_free_shipping = False
            availability_message = None

            # Check Offers (v1) - most reliable
            if hasattr(item, "offers") and item.offers:
                if hasattr(item.offers, "listings") and item.offers.listings:
                    listing = item.offers.listings[0]  # First listing

                    # Current price
                    if hasattr(listing, "price") and listing.price:
                        if hasattr(listing.price, "amount"):
                            current_price = float(listing.price.amount)
                        if hasattr(listing.price, "currency"):
                            currency = listing.price.currency

                    # List price / savings basis
                    if hasattr(listing, "saving_basis") and listing.saving_basis:
                        if hasattr(listing.saving_basis, "amount"):
                            list_price = float(listing.saving_basis.amount)

                    # Calculate savings
                    if current_price and list_price and list_price > current_price:
                        savings_amount = list_price - current_price
                        savings_percentage = int((savings_amount / list_price) * 100)

                    # Seller info
                    if hasattr(listing, "merchant_info") and listing.merchant_info:
                        if hasattr(listing.merchant_info, "name"):
                            seller_name = listing.merchant_info.name
                            if seller_name and "amazon" in seller_name.lower():
                                is_amazon_seller = True

                    # Availability
                    if hasattr(listing, "availability") and listing.availability:
                        if hasattr(listing.availability, "message"):
                            availability_message = listing.availability.message

                    # Delivery info
                    if hasattr(listing, "delivery_info") and listing.delivery_info:
                        if hasattr(listing.delivery_info, "is_prime_eligible"):
                            is_prime_eligible = listing.delivery_info.is_prime_eligible
                        if hasattr(listing.delivery_info, "is_free_shipping_eligible"):
                            is_free_shipping = listing.delivery_info.is_free_shipping_eligible

            # Must have valid price to record
            if current_price is None or current_price <= 0:
                logger.debug(f"  [{asin}] No valid price found, skipping")
                return None

            # Store price history record
            price_record = PriceHistory(
                asin=asin,
                price_amount=current_price,
                currency=currency,
                list_price=list_price,
                savings_amount=savings_amount,
                savings_percentage=savings_percentage,
                seller_name=seller_name,
                is_amazon_seller=is_amazon_seller,
                is_prime_eligible=is_prime_eligible,
                is_free_shipping=is_free_shipping,
                availability_message=availability_message,
                captured_at=datetime.now(UTC),
            )

            self.db_session.add(price_record)
            self.db_session.commit()

            logger.debug(f"  [{asin}] Price recorded: {current_price} {currency}")

            # Update price statistics
            self._update_price_statistics(asin)

            return price_record

        except Exception as e:
            logger.error(f"Error storing price for ASIN: {e}")
            self.db_session.rollback()
            return None

    def _update_price_statistics(self, asin: str):
        """
        Update materialized price statistics for an ASIN.

        Calculates min/max/avg prices over different time windows.

        Args:
            asin: Product ASIN
        """
        try:
            now = datetime.now(UTC)

            # Get current price
            latest = (
                self.db_session.query(PriceHistory)
                .filter(PriceHistory.asin == asin)
                .order_by(PriceHistory.captured_at.desc())
                .first()
            )

            if not latest:
                return

            # Calculate statistics over different time windows
            stats = {
                "asin": asin,
                "current_price": latest.price_amount,
                "current_price_updated_at": latest.captured_at,
            }

            # 24 hours
            stats_24h = self._calculate_window_stats(asin, now - timedelta(hours=24))
            stats.update(
                {
                    "min_price_24h": stats_24h["min_price"],
                    "max_price_24h": stats_24h["max_price"],
                }
            )

            # 7 days
            stats_7d = self._calculate_window_stats(asin, now - timedelta(days=7))
            stats.update(
                {
                    "min_price_7d": stats_7d["min_price"],
                    "max_price_7d": stats_7d["max_price"],
                    "avg_price_7d": stats_7d["avg_price"],
                }
            )

            # 30 days
            stats_30d = self._calculate_window_stats(asin, now - timedelta(days=30))
            stats.update(
                {
                    "min_price_30d": stats_30d["min_price"],
                    "max_price_30d": stats_30d["max_price"],
                    "avg_price_30d": stats_30d["avg_price"],
                    "price_volatility_30d": stats_30d["volatility"],
                }
            )

            # All time
            all_time = (
                self.db_session.query(func.min(PriceHistory.price_amount)).filter(PriceHistory.asin == asin).scalar()
            )
            stats["min_price_all_time"] = all_time

            # Price change 24h
            if stats_24h["first_price"]:
                stats["price_change_24h"] = float(latest.price_amount) - stats_24h["first_price"]
                stats["price_change_24h_pct"] = float((stats["price_change_24h"] / stats_24h["first_price"]) * 100)

            # Upsert statistics
            existing = self.db_session.query(PriceStatistics).filter_by(asin=asin).first()
            if existing:
                for key, value in stats.items():
                    if key != "asin":
                        setattr(existing, key, value)
                existing.updated_at = now
            else:
                stat_record = PriceStatistics(**stats)
                self.db_session.add(stat_record)

            self.db_session.commit()

            # SMART MONITORING: Update next_check_at based on activity
            self._update_adaptive_schedule(asin, stats)

        except Exception as e:
            logger.error(f"Error updating statistics for {asin}: {e}")
            self.db_session.rollback()

    def _update_adaptive_schedule(self, asin: str, stats: dict):
        """
        Dynamically schedule the next check time based on product activity.

        Strategies:
        1. High Priority (1h): Significant price change recently or deal detected.
        2. Medium Priority (4-6h): Stable price for 24h.
        3. Low Priority (12-24h): Stable for > 3 days or out of stock.
        """
        try:
            product = self.db_session.query(Product).filter(Product.asin == asin).first()
            if not product:
                return

            now = datetime.now(UTC)
            next_check_hours = 4  # Default medium
            priority = 2

            # Signal: High Demand / Popularity
            # If product has lots of reviews, it's popular -> Prices fluctuate more often -> Check more often
            # This is a 'Professional' touch: Volume = Volatility
            review_count = product.review_count or 0
            if review_count > 1000:
                next_check_hours = 3  # Boost standard check frequency
                logger.debug(f"  [{asin}] High popularity ({review_count} reviews). Reduced interval to 3h.")

            # Signal: Recent Price Change (Last 24h)
            pct_change = abs(stats.get("price_change_24h_pct") or 0)

            if pct_change > 1.0:
                # Active! Check sooner.
                next_check_hours = 1
                priority = 1
                logger.debug(f"  [{asin}] Active price change ({pct_change:.1f}%). High priority.")

            # Signal: Stable for long time
            elif (stats.get("price_volatility_30d") or 0) < 0.1:
                # Very stable. Check less often.
                next_check_hours = 12
                # But if it's super popular, don't ignore it for THAT long
                if review_count > 1000:
                    next_check_hours = 6

                priority = 3
                logger.debug(f"  [{asin}] Price stable. Low priority.")

            # Signal: High Savings (Potential Deal)
            # If current price is much lower than 30d max, keep watching closely
            max_30d = float(stats.get("max_price_30d") or 0)
            current = float(stats.get("current_price") or 0)
            if max_30d > 0 and current > 0:
                discount_from_high = ((max_30d - current) / max_30d) * 100
                if discount_from_high > 20:
                    # It's deeply discounted, watch for further drops or expiration
                    next_check_hours = 2
                    priority = 1

            # Update Product
            product.check_priority = priority
            product.next_check_at = now + timedelta(hours=next_check_hours)

            self.db_session.commit()

        except Exception as e:
            logger.error(f"Error updating schedule for {asin}: {e}")

    def _calculate_window_stats(self, asin: str, since: datetime) -> dict:
        """Calculate price statistics for a time window."""
        records = (
            self.db_session.query(PriceHistory.price_amount)
            .filter(and_(PriceHistory.asin == asin, PriceHistory.captured_at >= since))
            .order_by(PriceHistory.captured_at)
            .all()
        )

        if not records:
            return {
                "min_price": None,
                "max_price": None,
                "avg_price": None,
                "first_price": None,
                "volatility": None,
            }

        prices = [float(r[0]) for r in records]

        # Calculate volatility (standard deviation)
        volatility = None
        if len(prices) > 1:
            avg = sum(prices) / len(prices)
            variance = sum((p - avg) ** 2 for p in prices) / len(prices)
            volatility = variance**0.5

        return {
            "min_price": min(prices),
            "max_price": max(prices),
            "avg_price": sum(prices) / len(prices),
            "first_price": prices[0],
            "volatility": volatility,
        }



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six


class OfferShippingCharge(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "amount": "float",
        "currency": "str",
        "display_amount": "str",
        "is_rate_tax_inclusive": "bool",
        "type": "str",
    }

    attribute_map = {
        "amount": "Amount",
        "currency": "Currency",
        "display_amount": "DisplayAmount",
        "is_rate_tax_inclusive": "IsRateTaxInclusive",
        "type": "Type",
    }

    def __init__(
        self,
        amount=None,
        currency=None,
        display_amount=None,
        is_rate_tax_inclusive=None,
        type=None,
    ):  # noqa: E501
        """OfferShippingCharge - a model defined in Swagger"""  # noqa: E501

        self._amount = None
        self._currency = None
        self._display_amount = None
        self._is_rate_tax_inclusive = None
        self._type = None
        self.discriminator = None

        if amount is not None:
            self.amount = amount
        if currency is not None:
            self.currency = currency
        if display_amount is not None:
            self.display_amount = display_amount
        if is_rate_tax_inclusive is not None:
            self.is_rate_tax_inclusive = is_rate_tax_inclusive
        if type is not None:
            self.type = type

    @property
    def amount(self):
        """Gets the amount of this OfferShippingCharge.  # noqa: E501


        :return: The amount of this OfferShippingCharge.  # noqa: E501
        :rtype: float
        """
        return self._amount

    @amount.setter
    def amount(self, amount):
        """Sets the amount of this OfferShippingCharge.


        :param amount: The amount of this OfferShippingCharge.  # noqa: E501
        :type: float
        """

        self._amount = amount

    @property
    def currency(self):
        """Gets the currency of this OfferShippingCharge.  # noqa: E501


        :return: The currency of this OfferShippingCharge.  # noqa: E501
        :rtype: str
        """
        return self._currency

    @currency.setter
    def currency(self, currency):
        """Sets the currency of this OfferShippingCharge.


        :param currency: The currency of this OfferShippingCharge.  # noqa: E501
        :type: str
        """

        self._currency = currency

    @property
    def display_amount(self):
        """Gets the display_amount of this OfferShippingCharge.  # noqa: E501


        :return: The display_amount of this OfferShippingCharge.  # noqa: E501
        :rtype: str
        """
        return self._display_amount

    @display_amount.setter
    def display_amount(self, display_amount):
        """Sets the display_amount of this OfferShippingCharge.


        :param display_amount: The display_amount of this OfferShippingCharge.  # noqa: E501
        :type: str
        """

        self._display_amount = display_amount

    @property
    def is_rate_tax_inclusive(self):
        """Gets the is_rate_tax_inclusive of this OfferShippingCharge.  # noqa: E501


        :return: The is_rate_tax_inclusive of this OfferShippingCharge.  # noqa: E501
        :rtype: bool
        """
        return self._is_rate_tax_inclusive

    @is_rate_tax_inclusive.setter
    def is_rate_tax_inclusive(self, is_rate_tax_inclusive):
        """Sets the is_rate_tax_inclusive of this OfferShippingCharge.


        :param is_rate_tax_inclusive: The is_rate_tax_inclusive of this OfferShippingCharge.  # noqa: E501
        :type: bool
        """

        self._is_rate_tax_inclusive = is_rate_tax_inclusive

    @property
    def type(self):
        """Gets the type of this OfferShippingCharge.  # noqa: E501


        :return: The type of this OfferShippingCharge.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this OfferShippingCharge.


        :param type: The type of this OfferShippingCharge.  # noqa: E501
        :type: str
        """

        self._type = type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(OfferShippingCharge, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OfferShippingCharge):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.condition import Condition  # noqa: F401,E501
from paapi5_python_sdk.models.get_items_resource import GetItemsResource  # noqa: F401,E501
from paapi5_python_sdk.models.item_id_type import ItemIdType  # noqa: F401,E501
from paapi5_python_sdk.models.merchant import Merchant  # noqa: F401,E501
from paapi5_python_sdk.models.offer_count import OfferCount  # noqa: F401,E501
from paapi5_python_sdk.models.partner_type import PartnerType  # noqa: F401,E501
from paapi5_python_sdk.models.properties import Properties  # noqa: F401,E501


class GetItemsRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "condition": "Condition",
        "currency_of_preference": "str",
        "item_ids": "list[str]",
        "item_id_type": "ItemIdType",
        "languages_of_preference": "list[str]",
        "marketplace": "str",
        "merchant": "Merchant",
        "offer_count": "OfferCount",
        "partner_tag": "str",
        "partner_type": "PartnerType",
        "properties": "Properties",
        "resources": "list[GetItemsResource]",
    }

    attribute_map = {
        "condition": "Condition",
        "currency_of_preference": "CurrencyOfPreference",
        "item_ids": "ItemIds",
        "item_id_type": "ItemIdType",
        "languages_of_preference": "LanguagesOfPreference",
        "marketplace": "Marketplace",
        "merchant": "Merchant",
        "offer_count": "OfferCount",
        "partner_tag": "PartnerTag",
        "partner_type": "PartnerType",
        "properties": "Properties",
        "resources": "Resources",
    }

    def __init__(
        self,
        condition=None,
        currency_of_preference=None,
        item_ids=None,
        item_id_type=None,
        languages_of_preference=None,
        marketplace=None,
        merchant=None,
        offer_count=None,
        partner_tag=None,
        partner_type=None,
        properties=None,
        resources=None,
    ):  # noqa: E501
        """GetItemsRequest - a model defined in Swagger"""  # noqa: E501

        self._condition = None
        self._currency_of_preference = None
        self._item_ids = None
        self._item_id_type = None
        self._languages_of_preference = None
        self._marketplace = None
        self._merchant = None
        self._offer_count = None
        self._partner_tag = None
        self._partner_type = None
        self._properties = None
        self._resources = None
        self.discriminator = None

        if condition is not None:
            self.condition = condition
        if currency_of_preference is not None:
            self.currency_of_preference = currency_of_preference
        self.item_ids = item_ids
        if item_id_type is not None:
            self.item_id_type = item_id_type
        if languages_of_preference is not None:
            self.languages_of_preference = languages_of_preference
        if marketplace is not None:
            self.marketplace = marketplace
        if merchant is not None:
            self.merchant = merchant
        if offer_count is not None:
            self.offer_count = offer_count
        self.partner_tag = partner_tag
        self.partner_type = partner_type
        if properties is not None:
            self.properties = properties
        if resources is not None:
            self.resources = resources

    @property
    def condition(self):
        """Gets the condition of this GetItemsRequest.  # noqa: E501


        :return: The condition of this GetItemsRequest.  # noqa: E501
        :rtype: Condition
        """
        return self._condition

    @condition.setter
    def condition(self, condition):
        """Sets the condition of this GetItemsRequest.


        :param condition: The condition of this GetItemsRequest.  # noqa: E501
        :type: Condition
        """

        self._condition = condition

    @property
    def currency_of_preference(self):
        """Gets the currency_of_preference of this GetItemsRequest.  # noqa: E501


        :return: The currency_of_preference of this GetItemsRequest.  # noqa: E501
        :rtype: str
        """
        return self._currency_of_preference

    @currency_of_preference.setter
    def currency_of_preference(self, currency_of_preference):
        """Sets the currency_of_preference of this GetItemsRequest.


        :param currency_of_preference: The currency_of_preference of this GetItemsRequest.  # noqa: E501
        :type: str
        """

        self._currency_of_preference = currency_of_preference

    @property
    def item_ids(self):
        """Gets the item_ids of this GetItemsRequest.  # noqa: E501


        :return: The item_ids of this GetItemsRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._item_ids

    @item_ids.setter
    def item_ids(self, item_ids):
        """Sets the item_ids of this GetItemsRequest.


        :param item_ids: The item_ids of this GetItemsRequest.  # noqa: E501
        :type: list[str]
        """
        if item_ids is None:
            raise ValueError("Invalid value for `item_ids`, must not be `None`")  # noqa: E501

        self._item_ids = item_ids

    @property
    def item_id_type(self):
        """Gets the item_id_type of this GetItemsRequest.  # noqa: E501


        :return: The item_id_type of this GetItemsRequest.  # noqa: E501
        :rtype: ItemIdType
        """
        return self._item_id_type

    @item_id_type.setter
    def item_id_type(self, item_id_type):
        """Sets the item_id_type of this GetItemsRequest.


        :param item_id_type: The item_id_type of this GetItemsRequest.  # noqa: E501
        :type: ItemIdType
        """

        self._item_id_type = item_id_type

    @property
    def languages_of_preference(self):
        """Gets the languages_of_preference of this GetItemsRequest.  # noqa: E501


        :return: The languages_of_preference of this GetItemsRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._languages_of_preference

    @languages_of_preference.setter
    def languages_of_preference(self, languages_of_preference):
        """Sets the languages_of_preference of this GetItemsRequest.


        :param languages_of_preference: The languages_of_preference of this GetItemsRequest.  # noqa: E501
        :type: list[str]
        """

        self._languages_of_preference = languages_of_preference

    @property
    def marketplace(self):
        """Gets the marketplace of this GetItemsRequest.  # noqa: E501


        :return: The marketplace of this GetItemsRequest.  # noqa: E501
        :rtype: str
        """
        return self._marketplace

    @marketplace.setter
    def marketplace(self, marketplace):
        """Sets the marketplace of this GetItemsRequest.


        :param marketplace: The marketplace of this GetItemsRequest.  # noqa: E501
        :type: str
        """

        self._marketplace = marketplace

    @property
    def merchant(self):
        """Gets the merchant of this GetItemsRequest.  # noqa: E501


        :return: The merchant of this GetItemsRequest.  # noqa: E501
        :rtype: Merchant
        """
        return self._merchant

    @merchant.setter
    def merchant(self, merchant):
        """Sets the merchant of this GetItemsRequest.


        :param merchant: The merchant of this GetItemsRequest.  # noqa: E501
        :type: Merchant
        """

        self._merchant = merchant

    @property
    def offer_count(self):
        """Gets the offer_count of this GetItemsRequest.  # noqa: E501


        :return: The offer_count of this GetItemsRequest.  # noqa: E501
        :rtype: OfferCount
        """
        return self._offer_count

    @offer_count.setter
    def offer_count(self, offer_count):
        """Sets the offer_count of this GetItemsRequest.


        :param offer_count: The offer_count of this GetItemsRequest.  # noqa: E501
        :type: OfferCount
        """

        self._offer_count = offer_count

    @property
    def partner_tag(self):
        """Gets the partner_tag of this GetItemsRequest.  # noqa: E501


        :return: The partner_tag of this GetItemsRequest.  # noqa: E501
        :rtype: str
        """
        return self._partner_tag

    @partner_tag.setter
    def partner_tag(self, partner_tag):
        """Sets the partner_tag of this GetItemsRequest.


        :param partner_tag: The partner_tag of this GetItemsRequest.  # noqa: E501
        :type: str
        """
        if partner_tag is None:
            raise ValueError("Invalid value for `partner_tag`, must not be `None`")  # noqa: E501

        self._partner_tag = partner_tag

    @property
    def partner_type(self):
        """Gets the partner_type of this GetItemsRequest.  # noqa: E501


        :return: The partner_type of this GetItemsRequest.  # noqa: E501
        :rtype: PartnerType
        """
        return self._partner_type

    @partner_type.setter
    def partner_type(self, partner_type):
        """Sets the partner_type of this GetItemsRequest.


        :param partner_type: The partner_type of this GetItemsRequest.  # noqa: E501
        :type: PartnerType
        """
        if partner_type is None:
            raise ValueError("Invalid value for `partner_type`, must not be `None`")  # noqa: E501

        self._partner_type = partner_type

    @property
    def properties(self):
        """Gets the properties of this GetItemsRequest.  # noqa: E501


        :return: The properties of this GetItemsRequest.  # noqa: E501
        :rtype: Properties
        """
        return self._properties

    @properties.setter
    def properties(self, properties):
        """Sets the properties of this GetItemsRequest.


        :param properties: The properties of this GetItemsRequest.  # noqa: E501
        :type: Properties
        """

        self._properties = properties

    @property
    def resources(self):
        """Gets the resources of this GetItemsRequest.  # noqa: E501


        :return: The resources of this GetItemsRequest.  # noqa: E501
        :rtype: list[GetItemsResource]
        """
        return self._resources

    @resources.setter
    def resources(self, resources):
        """Sets the resources of this GetItemsRequest.


        :param resources: The resources of this GetItemsRequest.  # noqa: E501
        :type: list[GetItemsResource]
        """

        self._resources = resources

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(GetItemsRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GetItemsRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.offer_savings import OfferSavings  # noqa: F401,E501
from paapi5_python_sdk.models.price_type import PriceType  # noqa: F401,E501


class OfferPrice(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "amount": "float",
        "currency": "str",
        "display_amount": "str",
        "price_per_unit": "float",
        "price_type": "PriceType",
        "price_type_label": "str",
        "savings": "OfferSavings",
    }

    attribute_map = {
        "amount": "Amount",
        "currency": "Currency",
        "display_amount": "DisplayAmount",
        "price_per_unit": "PricePerUnit",
        "price_type": "PriceType",
        "price_type_label": "PriceTypeLabel",
        "savings": "Savings",
    }

    def __init__(
        self,
        amount=None,
        currency=None,
        display_amount=None,
        price_per_unit=None,
        price_type=None,
        price_type_label=None,
        savings=None,
    ):  # noqa: E501
        """OfferPrice - a model defined in Swagger"""  # noqa: E501

        self._amount = None
        self._currency = None
        self._display_amount = None
        self._price_per_unit = None
        self._price_type = None
        self._price_type_label = None
        self._savings = None
        self.discriminator = None

        if amount is not None:
            self.amount = amount
        if currency is not None:
            self.currency = currency
        if display_amount is not None:
            self.display_amount = display_amount
        if price_per_unit is not None:
            self.price_per_unit = price_per_unit
        if price_type is not None:
            self.price_type = price_type
        if price_type_label is not None:
            self.price_type_label = price_type_label
        if savings is not None:
            self.savings = savings

    @property
    def amount(self):
        """Gets the amount of this OfferPrice.  # noqa: E501


        :return: The amount of this OfferPrice.  # noqa: E501
        :rtype: float
        """
        return self._amount

    @amount.setter
    def amount(self, amount):
        """Sets the amount of this OfferPrice.


        :param amount: The amount of this OfferPrice.  # noqa: E501
        :type: float
        """

        self._amount = amount

    @property
    def currency(self):
        """Gets the currency of this OfferPrice.  # noqa: E501


        :return: The currency of this OfferPrice.  # noqa: E501
        :rtype: str
        """
        return self._currency

    @currency.setter
    def currency(self, currency):
        """Sets the currency of this OfferPrice.


        :param currency: The currency of this OfferPrice.  # noqa: E501
        :type: str
        """

        self._currency = currency

    @property
    def display_amount(self):
        """Gets the display_amount of this OfferPrice.  # noqa: E501


        :return: The display_amount of this OfferPrice.  # noqa: E501
        :rtype: str
        """
        return self._display_amount

    @display_amount.setter
    def display_amount(self, display_amount):
        """Sets the display_amount of this OfferPrice.


        :param display_amount: The display_amount of this OfferPrice.  # noqa: E501
        :type: str
        """

        self._display_amount = display_amount

    @property
    def price_per_unit(self):
        """Gets the price_per_unit of this OfferPrice.  # noqa: E501


        :return: The price_per_unit of this OfferPrice.  # noqa: E501
        :rtype: float
        """
        return self._price_per_unit

    @price_per_unit.setter
    def price_per_unit(self, price_per_unit):
        """Sets the price_per_unit of this OfferPrice.


        :param price_per_unit: The price_per_unit of this OfferPrice.  # noqa: E501
        :type: float
        """

        self._price_per_unit = price_per_unit

    @property
    def price_type(self):
        """Gets the price_type of this OfferPrice.  # noqa: E501


        :return: The price_type of this OfferPrice.  # noqa: E501
        :rtype: PriceType
        """
        return self._price_type

    @price_type.setter
    def price_type(self, price_type):
        """Sets the price_type of this OfferPrice.


        :param price_type: The price_type of this OfferPrice.  # noqa: E501
        :type: PriceType
        """

        self._price_type = price_type

    @property
    def price_type_label(self):
        """Gets the price_type_label of this OfferPrice.  # noqa: E501


        :return: The price_type_label of this OfferPrice.  # noqa: E501
        :rtype: str
        """
        return self._price_type_label

    @price_type_label.setter
    def price_type_label(self, price_type_label):
        """Sets the price_type_label of this OfferPrice.


        :param price_type_label: The price_type_label of this OfferPrice.  # noqa: E501
        :type: str
        """

        self._price_type_label = price_type_label

    @property
    def savings(self):
        """Gets the savings of this OfferPrice.  # noqa: E501


        :return: The savings of this OfferPrice.  # noqa: E501
        :rtype: OfferSavings
        """
        return self._savings

    @savings.setter
    def savings(self, savings):
        """Sets the savings of this OfferPrice.


        :param savings: The savings of this OfferPrice.  # noqa: E501
        :type: OfferSavings
        """

        self._savings = savings

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(OfferPrice, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OfferPrice):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Database models for Opportunity Radar."""

from opportunity_core.models.database import (
    AlertSent,
    Base,
    PriceHistory,
    PriceStatistics,
    Product,
)

__all__ = [
    "Base",
    "Product",
    "PriceHistory",
    "PriceStatistics",
    "AlertSent",
]



"""Database connection and session management."""

import logging
from contextlib import contextmanager
from typing import Generator

from sqlalchemy import create_engine, event
from sqlalchemy.engine import Engine
from sqlalchemy.orm import Session, sessionmaker
from sqlalchemy.pool import QueuePool

from opportunity_core.models.database import Base

logger = logging.getLogger(__name__)


class DatabaseManager:
    """Manages database connections and sessions."""

    def __init__(self, database_url: str, echo: bool = False, pool_size: int = 10, max_overflow: int = 20):
        """
        Initialize database manager.

        Args:
            database_url: SQLAlchemy database URL (e.g., postgresql://user:pass@host:port/db)
            echo: Whether to echo SQL queries (for debugging)
            pool_size: Number of connections to keep in pool
            max_overflow: Maximum overflow connections beyond pool_size
        """
        self.database_url = database_url
        self.engine = create_engine(
            database_url,
            echo=echo,
            poolclass=QueuePool,
            pool_size=pool_size,
            max_overflow=max_overflow,
            pool_pre_ping=True,  # Verify connections before using
            pool_recycle=3600,  # Recycle connections after 1 hour
        )

        # Configure SQLite for better concurrency if using SQLite
        if "sqlite" in database_url.lower():

            @event.listens_for(Engine, "connect")
            def set_sqlite_pragma(dbapi_conn, connection_record):
                cursor = dbapi_conn.cursor()
                cursor.execute("PRAGMA journal_mode=WAL")
                cursor.execute("PRAGMA synchronous=NORMAL")
                cursor.execute("PRAGMA foreign_keys=ON")
                cursor.close()

        self.SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=self.engine)
        logger.info(f"Database manager initialized: {self._safe_url()}")

    def _safe_url(self) -> str:
        """Return database URL with password masked."""
        if "@" in self.database_url:
            parts = self.database_url.split("@")
            if ":" in parts[0]:
                user_pass = parts[0].split("://")[-1]
                user = user_pass.split(":")[0] if ":" in user_pass else user_pass
                return f"{self.database_url.split('://')[0]}://{user}:***@{parts[1]}"
        return self.database_url

    def create_tables(self):
        """Create all tables in the database."""
        logger.info("Creating database tables...")
        Base.metadata.create_all(bind=self.engine)
        logger.info("Database tables created successfully")

    def drop_tables(self):
        """Drop all tables in the database (USE WITH CAUTION)."""
        logger.warning("Dropping all database tables...")
        Base.metadata.drop_all(bind=self.engine)
        logger.warning("All database tables dropped")

    @contextmanager
    def get_session(self) -> Generator[Session, None, None]:
        """
        Get a database session with automatic commit/rollback.

        Usage:
            with db_manager.get_session() as session:
                product = session.query(Product).filter_by(asin="B08TEST").first()
        """
        session = self.SessionLocal()
        try:
            yield session
            session.commit()
        except Exception as e:
            session.rollback()
            logger.error(f"Database session error: {e}")
            raise
        finally:
            session.close()

    def get_raw_session(self) -> Session:
        """
        Get a raw database session (caller must manage commit/rollback/close).

        Use get_session() context manager instead when possible.
        """
        return self.SessionLocal()

    def health_check(self) -> bool:
        """
        Check if database connection is healthy.

        Returns:
            True if connection successful, False otherwise
        """
        try:
            with self.get_session() as session:
                session.execute("SELECT 1")
            return True
        except Exception as e:
            logger.error(f"Database health check failed: {e}")
            return False

    def close(self):
        """Close database engine and dispose of connection pool."""
        logger.info("Closing database connections...")
        self.engine.dispose()
        logger.info("Database connections closed")


# Singleton instance for application use
_db_manager: DatabaseManager | None = None


def init_database(database_url: str, echo: bool = False) -> DatabaseManager:
    """
    Initialize the global database manager.

    Args:
        database_url: SQLAlchemy database URL
        echo: Whether to echo SQL queries

    Returns:
        DatabaseManager instance
    """
    global _db_manager
    _db_manager = DatabaseManager(database_url, echo=echo)
    return _db_manager


def get_db_manager() -> DatabaseManager:
    """
    Get the global database manager instance.

    Raises:
        RuntimeError: If database not initialized
    """
    if _db_manager is None:
        raise RuntimeError("Database not initialized. Call init_database() first.")
    return _db_manager


# Dependency injection helper for FastAPI / web frameworks
def get_db() -> Generator[Session, None, None]:
    """
    FastAPI dependency for getting database session.

    Usage:
        @app.get("/products")
        def get_products(db: Session = Depends(get_db)):
            return db.query(Product).all()
    """
    db_manager = get_db_manager()
    with db_manager.get_session() as session:
        yield session
